/*
 * Drawing instructions for Weather icons
 */

#include "icons.h"

// CS and RST assignments
#define RA8875_CS 5
#define RA8875_RESET 33
#define RED 0xF082
#define GREEN 0x4DA2
#define OFF_WHITE 0xEF5D
#define GRAY 0xAD34
#define LIGHT_BROWN 0xCCAC
#define LIGHTER_BROWN 0xEDD0
#define DARKER_BROWN 0x72E8
#define LIGHT_GREEN 0x8E25
#define LIGHT_ORANGE 0xFDEC
#define YELLOW 0xFF0E

//Adafruit_RA8875 tft = Adafruit_RA8875(RA8875_CS, RA8875_RESET);

void iconCloud(Adafruit_RA8875& tft, uint16_t x, uint16_t y, uint16_t r)
{
  // top circle
  tft.fillCircle(x, y, r, RA8875_BLACK);
  // left circle
  tft.fillCircle(x-r*0.85, y+r*0.8, r*0.85, RA8875_BLACK);
  // right circle
  tft.fillCircle(x+r*1.1, y+r*0.8, r*0.85, RA8875_BLACK);
  // rectangle
  tft.fillRect(x-r*0.85, y+r*0.8, (x+r*1.1)- (x-r*0.85), r*0.9, RA8875_BLACK );

  // top circle
  float offset = 0.8;
  tft.fillCircle(x, y, r*offset, RA8875_WHITE);
  // left circle
  tft.fillCircle(x-r*0.85, y+r*0.8, r*0.85*offset, RA8875_WHITE);
  // right circle
  tft.fillCircle(x+r*1.1, y+r*0.8, r*0.85*offset, RA8875_WHITE);
  // rectangle
  tft.fillRect(x-r*0.85, y+r*0.7, (x+r*1.1) - (x-r*0.85), r*offset, RA8875_WHITE );

}

void iconSun(Adafruit_RA8875& tft, uint16_t x, uint16_t y, uint16_t r)
{
  tft.drawLine(x-r*1.75, y, x+r*1.75,y, LIGHT_ORANGE);
  tft.drawLine(x, y-r*1.75, x, y+r*1.75, LIGHT_ORANGE);
  tft.drawLine(x - r*1.25, y - r*1.25, x + r*1.25, y +r*1.25, LIGHT_ORANGE);
  tft.drawLine(x - r*1.25, y + r*1.25, x + r*1.25, y - r*1.25, LIGHT_ORANGE);
  tft.fillCircle(x, y, r*1.2, LIGHT_BROWN);
  tft.fillCircle(x, y, r*1.1, RA8875_WHITE);
  tft.fillCircle(x, y, r, LIGHT_ORANGE);
  float offset = 0.9;
  tft.fillCircle(x, y, r*offset, YELLOW);
}

void iconSunSmile(Adafruit_RA8875& tft, uint16_t x, uint16_t y, uint16_t r)
{
  tft.drawLine(x-r*1.75, y, x+r*1.75,y, LIGHT_ORANGE);
  tft.drawLine(x, y-r*1.75, x, y+r*1.75, LIGHT_ORANGE);
  tft.drawLine(x - r*1.25, y - r*1.25, x + r*1.25, y +r*1.25, LIGHT_ORANGE);
  tft.drawLine(x - r*1.25, y + r*1.25, x + r*1.25, y - r*1.25, LIGHT_ORANGE);
  tft.fillCircle(x, y, r*1.2, LIGHT_BROWN);
  tft.fillCircle(x, y, r*1.1, RA8875_WHITE);
  tft.fillCircle(x, y, r, LIGHT_ORANGE);
  float offset = 0.9;
  tft.fillCircle(x, y, r*offset, YELLOW);

  // eyes
  tft.fillRect();
}

// Edit moon
void iconMoon(Adafruit_RA8875& tft, uint16_t x, uint16_t y, uint16_t r)
{
  float offset = 0.9;
  tft.fillCircle(x, y, r, RA8875_BLACK);
  tft.fillCircle(x, y, r*offset, RA8875_RED);
  tft.fillCircle(x+r, y-r, r, RA8875_BLACK);
  tft.fillCircle(x+r, y-r, r*offset, RA8875_WHITE);
  tft.fillRect(x, y-r*2, r*2.5, r, RA8875_WHITE);
  tft.fillRect(x+r+1, y-r, r*1.5, r*1.5, RA8875_WHITE);
}

void iconClearDay(Adafruit_RA8875& tft, uint16_t x, uint16_t y, uint16_t s)
{
  iconSun(tft, x+s/2, y+s/2, s/5);
}

void iconClearNight(Adafruit_RA8875& tft, uint16_t x, uint16_t y, uint16_t s)
{
  iconMoon(tft, x+s/2, y+s/2, s/5);
}

void iconRain(Adafruit_RA8875& tft, uint16_t x, uint16_t y, uint16_t s)
{
  iconCloud(tft, x+s/2.2, y+s/2.5, s/5);
  tft.fillRect(x+s*0.275, y+s*0.6, s/2.5, s/5, RA8875_WHITE);

  float offset = 0.8;
  for (int i = 0; i <= s*0.1; i++)
  {
    tft.fillCircle( x+s*0.4-i*0.5, y+s*0.65+i, s*0.02, RA8875_BLACK);
    tft.fillCircle( x+s*0.6-i*0.5, y+s*0.65+i, s*0.02, RA8875_BLACK);
  }
  for (int i = 0; i <= s*0.16; i++)
  {
    tft.fillCircle( x+s*0.5-i*0.5, y+s*0.65+i, s*0.02, RA8875_BLACK);
  }
}

void iconSleet(Adafruit_RA8875& tft, uint16_t x, uint16_t y, uint16_t s)
{
  iconCloud(tft, x+s/2.2, y+s/2.5, s/5);
  tft.fillRect(x+s*0.275, y+s*0.6, s/2.5, s/5, RA8875_WHITE);

  float offset = 0.8;
  for (int i = 0; i <= s*0.1; i++)
  {
    if(i < 1 || i > s*0.1 *0.5 )
    {
      tft.fillCircle( x+s*0.4-i*0.5, y+s*0.65+i, s*0.02, RA8875_BLACK);
      tft.fillCircle( x+s*0.6-i*0.5, y+s*0.65+i, s*0.02, RA8875_BLACK);
    }
  }
  for (int i = 0; i <= s*0.16; i++)
  {
    if(i < s*0.16 * 0.5 || i > s*0.16 * 0.8 )
      tft.fillCircle( x+s*0.5-i*0.5, y+s*0.65+i, s*0.02, RA8875_BLACK);
  }
}

void iconSnow(Adafruit_RA8875& tft, uint16_t x, uint16_t y, uint16_t s)
{
  iconCloud(tft, x+s/2.2, y+s/2.5, s/5);
  tft.fillRect(x+s*0.275, y+s*0.6, s/2.5, s/5, RA8875_WHITE);

  float offset = 0.8;
  tft.fillCircle( x+s/2.75, y+s*0.7, s*0.02, RA8875_BLACK);
  tft.fillCircle( x+s/1.75, y+s*0.7, s*0.02, RA8875_BLACK);

  tft.fillCircle( x+s/2.75, y+s*0.8, s*0.02, RA8875_BLACK);
  tft.fillCircle( x+s/1.75, y+s*0.8, s*0.02, RA8875_BLACK);

  tft.fillCircle( x+s/2.15, y+s*0.65, s*0.02, RA8875_BLACK);
  tft.fillCircle( x+s/2.15, y+s*0.75, s*0.02, RA8875_BLACK);
  tft.fillCircle( x+s/2.15, y+s*0.85, s*0.02, RA8875_BLACK);

}

void iconWind(Adafruit_RA8875& tft, uint16_t x, uint16_t y, uint16_t s)
{
  float offset = 0.8;
  for (int i = 0; i <= s*0.7; i++)
  {
    if(i < s*0.6)
      tft.fillCircle( x+s*0.15+i, y+s*0.4, s*0.02, RA8875_BLACK);
    if(i < s*0.5)
      tft.fillCircle( x+s*0.1+i, y+s*0.5, s*0.02, RA8875_BLACK);
    if(i < s*0.2)
      tft.fillCircle( x+s*0.7+i, y+s*0.5, s*0.02, RA8875_BLACK);
    if(i < s*0.6)
      tft.fillCircle( x+s*0.2+i, y+s*0.6, s*0.02, RA8875_BLACK);
  }
}

void iconFog(Adafruit_RA8875& tft, uint16_t x, uint16_t y, uint16_t s)
{
  iconCloud(tft, x+s/2.2, y+s/2.5, s/5);
  tft.fillRect(x+s*0.1, y+s*0.55, s*0.75, s/5, RA8875_WHITE);

  float offset = 0.8;
  for (int i = 0; i <= s*0.7; i++)
  {
    tft.fillCircle( x+s*0.1+i, y+s*0.6, s*0.02, RA8875_BLACK);
    tft.fillCircle( x+s*0.2+i, y+s*0.7, s*0.02, RA8875_BLACK);
    tft.fillCircle( x+s*0.15+i, y+s*0.8, s*0.02, RA8875_BLACK);
  }
}

void iconCloudy(Adafruit_RA8875& tft, uint16_t x, uint16_t y, uint16_t s)
{
  iconCloud(tft, x+(s/4)*3, y+s/4, s/10);
  iconCloud(tft, x+s/2.1, y+s/2.2, s/5);
}

void iconCloudyDay(Adafruit_RA8875& tft, uint16_t x, uint16_t y, uint16_t s)
{
  iconSun(tft, x+(s/3)*2, y+s/2.5, s/6);
  iconCloud(tft, x+s/2.2, y+s/2.2, s/5);
}

void iconCloudyNight(Adafruit_RA8875& tft, uint16_t x, uint16_t y, uint16_t s)
{
  iconMoon(tft, x+(s/3)*2, y+s/3, s/6);
  iconCloud(tft, x+s/2.2, y+s/2.2, s/5);
}

void iconHail(Adafruit_RA8875& tft, uint16_t x, uint16_t y, uint16_t s)
{
  iconCloud(tft, x+s/2.2, y+s/2.5, s/5);
  tft.fillRect(x+s*0.275, y+s*0.6, s/2.5, s/5, RA8875_WHITE);

  float offset = 0.8;
  for (int i = 0; i <= s*0.1; i++)
  {
    if(i < s*0.1 *0.5 || i == s*0.1 )
    {
      tft.fillCircle( x+s*0.4-i*0.5, y+s*0.65+i, s*0.02, RA8875_BLACK);
      tft.fillCircle( x+s*0.6-i*0.5, y+s*0.65+i, s*0.02, RA8875_BLACK);
    }
  }
  for (int i = 0; i <= s*0.16; i++)
  {
    if(i < s*0.16 *0.7 || i == s*0.16 )
      tft.fillCircle( x+s*0.5-i*0.5, y+s*0.65+i, s*0.02, RA8875_BLACK);
  }
}

void iconThunderstorm(Adafruit_RA8875& tft, uint16_t x, uint16_t y, uint16_t s)
{
  iconCloud(tft, x+s/2.2, y+s/2.5, s/5);
  tft.fillRect(x+s*0.275, y+s*0.6, s/2.5, s/5, RA8875_WHITE);

  float offset = 0.8;
  for (int i = 0; i <= s*0.1; i++)
  {
    tft.fillCircle( x+s*0.6-i*0.5, y+s*0.65+i, s*0.02, RA8875_BLACK);
  }
  for (int i = 0; i <= s*0.16; i++)
  {
    tft.fillCircle( x+s*0.5-i*0.5, y+s*0.65+i, s*0.02, RA8875_BLACK);
  }
  tft.fillTriangle( x+s*0.3, y+s*0.75, x+s*0.325, y+s*0.65, x+s*0.375, y+s*0.65, RA8875_RED);
  tft.fillTriangle( x+s*0.3, y+s*0.75, x+s*0.4, y+s*0.7, x+s*0.33, y+s*0.7, RA8875_RED);
  tft.fillTriangle( x+s*0.3, y+s*0.85, x+s*0.35, y+s*0.7, x+s*0.4, y+s*0.7, RA8875_RED);

}

void fillEllipsis(Adafruit_RA8875& tft, uint16_t x, uint16_t y, uint16_t w, uint16_t h, uint16_t c)
{
  for(int yi=-h; yi<=h; yi++) {
    for(int xi=-w; xi<=w; xi++) {
        if(xi*xi*h*h+yi*yi*w*w <= h*h*w*w)
            tft.writePixel(x+xi, y+yi, c);
    }
  }
}

void iconTornado(Adafruit_RA8875& tft, uint16_t x, uint16_t y, uint16_t s)
{
  // 1
  fillEllipsis(tft, x+s*0.33, y+s*0.7, s/12*1.2, s/18*1.2, RA8875_BLACK);
  fillEllipsis(tft, x+s*0.33, y+s*0.7, s/12, s/18, RA8875_WHITE);
  // 2
  fillEllipsis(tft, x+s*0.32, y+s*0.65, s/9*1.2, s/16*1.2, RA8875_BLACK);
  fillEllipsis(tft, x+s*0.32, y+s*0.65, s/9, s/16, RA8875_WHITE);
  // 3
  fillEllipsis(tft, x+s*0.35, y+s*0.55, s/7*1.2, s/12*1.2, RA8875_BLACK);
  fillEllipsis(tft, x+s*0.35, y+s*0.55, s/7, s/12, RA8875_WHITE);
  // 4
  fillEllipsis(tft, x+s*0.425, y+s*0.425, s/5*1.2, s/8*1.2, RA8875_BLACK);
  fillEllipsis(tft, x+s*0.425, y+s*0.425, s/5, s/8, RA8875_WHITE);
  // 5
  fillEllipsis(tft, x+s*0.5, y+s*0.3, s/4*1.2, s/7*1.2, RA8875_BLACK);
  fillEllipsis(tft, x+s*0.5, y+s*0.3, s/4, s/7, RA8875_WHITE);
}

void iconSunRise(Adafruit_RA8875& tft, uint16_t x, uint16_t y, String direction)
{
  uint16_t r = 7;

  // Horizontal
  tft.drawLine(x - r*2 + 2, y, x + r*2 - 2, y, RA8875_BLACK);
  // Vertical
  tft.drawLine(x, y - r*2 + 2, x, y, RA8875_BLACK);
  // Angle Top right
  tft.drawLine(x - r*2 + 5, y - r*2 + 5, x, y, RA8875_BLACK);
  // Angle Top left
  tft.drawLine(x, y, x + r*2 - 5, y - r*2 + 5, RA8875_BLACK);
  // Remove lines inside
  tft.fillCircle(x, y, r + 1, RA8875_WHITE);
  // Empty inside
  tft.fillCircle(x, y, r-1, RA8875_RED);
  tft.drawCircle(x, y, r-1, RA8875_BLACK);
  // Overwrite the bottom
  tft.fillRect(x - r, y + 4, r*2, r, RA8875_WHITE);
  
  // Arrow up
  if(direction == "up")
  {
    tft.fillTriangle( x-r/2-1, y+r-2, x, y+r - 7, x+r/2+1, y+r-2, RA8875_WHITE);
    tft.drawLine( x-r/2, y+r-2, x, y+r - 6, RA8875_BLACK);
    tft.drawLine( x, y+r - 6, x+r/2, y+r-2, RA8875_BLACK);
  }
  // Arrow DOWN
  if(direction == "down")
  {
    tft.drawLine( x-r/2, y+r-2, x, y+r + 2, RA8875_BLACK);
    tft.drawLine( x, y+r + 2, x+r/2, y+r-2, RA8875_BLACK);
  }
  // Horizon line
  tft.drawLine( x-r, y+r-2, x-r/2, y+r-2, RA8875_BLACK);
  tft.drawLine( x+r/2, y+r-2, x+r, y+r-2, RA8875_BLACK);
}

// 0 = new moon
// 0.25 to a first quarter moon, 
// 0.5 to a full moon,
// 0.75 to a last quarter moon. 
void iconMoonPhase(Adafruit_RA8875& tft, uint16_t x, uint16_t y, uint16_t r, float phase)
{
  tft.fillCircle(x, y, r, RA8875_RED);
  if(phase == 0)
    tft.fillCircle(x, y, r, RA8875_BLACK);
  else if(phase > 0 && phase < 0.5)
  {
    for (int i=0; i<r+1; i++ )
    {
      float cx = sqrt( (r * r) - (i*i) );
      float c2 = cx * 2 * (1-(phase*2));
      tft.drawLine(x-cx,y+i, x-cx+c2, y+i, RA8875_BLACK);
      tft.drawLine(x-cx,y-i, x-cx+c2, y-i, RA8875_BLACK);
    }
  }
  else if(phase == 0.5)
    tft.fillCircle(x, y, r, RA8875_RED);
  else
  {
    tft.fillCircle(x, y, r, RA8875_BLACK);
    for (int i=0; i<r+1; i++ )
    {
      float cx = sqrt( (r * r) - (i*i) );
      float c2 = cx * 2 * ((1-phase)*2);
      tft.drawLine(x-cx,y+i, x-cx+c2, y+i, RA8875_RED);
      tft.drawLine(x-cx,y-i, x-cx+c2, y-i, RA8875_RED);
    }
  }

  // Add moon surface on top
  tft.drawPixel(x-r+22, y-r+1, RA8875_BLACK);
  tft.drawPixel(x-r+12, y-r+3, RA8875_BLACK);
  tft.drawPixel(x-r+24, y-r+3, RA8875_BLACK);
  tft.drawPixel(x-r+13, y-r+4, RA8875_BLACK);
  tft.drawPixel(x-r+15, y-r+4, RA8875_BLACK);
  tft.drawPixel(x-r+17, y-r+4, RA8875_BLACK);
  tft.drawPixel(x-r+24, y-r+4, RA8875_BLACK);
  tft.drawPixel(x-r+26, y-r+4, RA8875_BLACK);
  tft.drawPixel(x-r+10, y-r+5, RA8875_BLACK);
  tft.drawPixel(x-r+17, y-r+5, RA8875_BLACK);
  tft.drawPixel(x-r+26, y-r+5, RA8875_BLACK);
  tft.drawPixel(x-r+15, y-r+6, RA8875_BLACK);
  tft.drawPixel(x-r+17, y-r+6, RA8875_BLACK);
  tft.drawPixel(x-r+19, y-r+6, RA8875_BLACK);
  tft.drawPixel(x-r+7, y-r+7, RA8875_BLACK);
  tft.drawPixel(x-r+14, y-r+7, RA8875_BLACK);
  tft.drawPixel(x-r+16, y-r+7, RA8875_BLACK);
  tft.drawPixel(x-r+18, y-r+7, RA8875_BLACK);
  tft.drawPixel(x-r+19, y-r+7, RA8875_BLACK);
  tft.drawPixel(x-r+23, y-r+7, RA8875_BLACK);
  tft.drawPixel(x-r+25, y-r+7, RA8875_BLACK);
  tft.drawPixel(x-r+28, y-r+7, RA8875_BLACK);
  tft.drawPixel(x-r+30, y-r+7, RA8875_BLACK);
  tft.drawPixel(x-r+6, y-r+8, RA8875_BLACK);
  tft.drawPixel(x-r+9, y-r+8, RA8875_BLACK);
  tft.drawPixel(x-r+12, y-r+8, RA8875_BLACK);
  tft.drawPixel(x-r+15, y-r+8, RA8875_BLACK);
  tft.drawPixel(x-r+17, y-r+8, RA8875_BLACK);
  tft.drawPixel(x-r+19, y-r+8, RA8875_BLACK);
  tft.drawPixel(x-r+20, y-r+8, RA8875_BLACK);
  tft.drawPixel(x-r+21, y-r+8, RA8875_BLACK);
  tft.drawPixel(x-r+23, y-r+8, RA8875_BLACK);
  tft.drawPixel(x-r+26, y-r+8, RA8875_BLACK);
  tft.drawPixel(x-r+28, y-r+8, RA8875_BLACK);
  tft.drawPixel(x-r+29, y-r+8, RA8875_BLACK);
  tft.drawPixel(x-r+31, y-r+8, RA8875_BLACK);
  tft.drawPixel(x-r+8, y-r+9, RA8875_BLACK);
  tft.drawPixel(x-r+10, y-r+9, RA8875_BLACK);
  tft.drawPixel(x-r+14, y-r+9, RA8875_BLACK);
  tft.drawPixel(x-r+15, y-r+9, RA8875_BLACK);
  tft.drawPixel(x-r+17, y-r+9, RA8875_BLACK);
  tft.drawPixel(x-r+18, y-r+9, RA8875_BLACK);
  tft.drawPixel(x-r+20, y-r+9, RA8875_BLACK);
  tft.drawPixel(x-r+22, y-r+9, RA8875_BLACK);
  tft.drawPixel(x-r+23, y-r+9, RA8875_BLACK);
  tft.drawPixel(x-r+25, y-r+9, RA8875_BLACK);
  tft.drawPixel(x-r+26, y-r+9, RA8875_BLACK);
  tft.drawPixel(x-r+28, y-r+9, RA8875_BLACK);
  tft.drawPixel(x-r+29, y-r+9, RA8875_BLACK);
  tft.drawPixel(x-r+30, y-r+9, RA8875_BLACK);
  tft.drawPixel(x-r+6, y-r+10, RA8875_BLACK);
  tft.drawPixel(x-r+8, y-r+10, RA8875_BLACK);
  tft.drawPixel(x-r+11, y-r+10, RA8875_BLACK);
  tft.drawPixel(x-r+15, y-r+10, RA8875_BLACK);
  tft.drawPixel(x-r+16, y-r+10, RA8875_BLACK);
  tft.drawPixel(x-r+18, y-r+10, RA8875_BLACK);
  tft.drawPixel(x-r+20, y-r+10, RA8875_BLACK);
  tft.drawPixel(x-r+23, y-r+10, RA8875_BLACK);
  tft.drawPixel(x-r+24, y-r+10, RA8875_BLACK);
  tft.drawPixel(x-r+26, y-r+10, RA8875_BLACK);
  tft.drawPixel(x-r+29, y-r+10, RA8875_BLACK);
  tft.drawPixel(x-r+6, y-r+11, RA8875_BLACK);
  tft.drawPixel(x-r+7, y-r+11, RA8875_BLACK);
  tft.drawPixel(x-r+9, y-r+11, RA8875_BLACK);
  tft.drawPixel(x-r+10, y-r+11, RA8875_BLACK);
  tft.drawPixel(x-r+12, y-r+11, RA8875_BLACK);
  tft.drawPixel(x-r+16, y-r+11, RA8875_BLACK);
  tft.drawPixel(x-r+18, y-r+11, RA8875_BLACK);
  tft.drawPixel(x-r+21, y-r+11, RA8875_BLACK);
  tft.drawPixel(x-r+24, y-r+11, RA8875_BLACK);
  tft.drawPixel(x-r+28, y-r+11, RA8875_BLACK);
  tft.drawPixel(x-r+30, y-r+11, RA8875_BLACK);
  tft.drawPixel(x-r+31, y-r+11, RA8875_BLACK);
  tft.drawPixel(x-r+3, y-r+12, RA8875_BLACK);
  tft.drawPixel(x-r+5, y-r+12, RA8875_BLACK);
  tft.drawPixel(x-r+7, y-r+12, RA8875_BLACK);
  tft.drawPixel(x-r+9, y-r+12, RA8875_BLACK);
  tft.drawPixel(x-r+11, y-r+12, RA8875_BLACK);
  tft.drawPixel(x-r+16, y-r+12, RA8875_BLACK);
  tft.drawPixel(x-r+19, y-r+12, RA8875_BLACK);
  tft.drawPixel(x-r+21, y-r+12, RA8875_BLACK);
  tft.drawPixel(x-r+23, y-r+12, RA8875_BLACK);
  tft.drawPixel(x-r+24, y-r+12, RA8875_BLACK);
  tft.drawPixel(x-r+25, y-r+12, RA8875_BLACK);
  tft.drawPixel(x-r+32, y-r+12, RA8875_BLACK);
  tft.drawPixel(x-r+5, y-r+13, RA8875_BLACK);
  tft.drawPixel(x-r+7, y-r+13, RA8875_BLACK);
  tft.drawPixel(x-r+8, y-r+13, RA8875_BLACK);
  tft.drawPixel(x-r+10, y-r+13, RA8875_BLACK);
  tft.drawPixel(x-r+13, y-r+13, RA8875_BLACK);
  tft.drawPixel(x-r+17, y-r+13, RA8875_BLACK);
  tft.drawPixel(x-r+22, y-r+13, RA8875_BLACK);
  tft.drawPixel(x-r+1, y-r+14, RA8875_BLACK);
  tft.drawPixel(x-r+5, y-r+14, RA8875_BLACK);
  tft.drawPixel(x-r+6, y-r+14, RA8875_BLACK);
  tft.drawPixel(x-r+8, y-r+14, RA8875_BLACK);
  tft.drawPixel(x-r+10, y-r+14, RA8875_BLACK);
  tft.drawPixel(x-r+12, y-r+14, RA8875_BLACK);
  tft.drawPixel(x-r+14, y-r+14, RA8875_BLACK);
  tft.drawPixel(x-r+17, y-r+14, RA8875_BLACK);
  tft.drawPixel(x-r+18, y-r+14, RA8875_BLACK);
  tft.drawPixel(x-r+25, y-r+14, RA8875_BLACK);
  tft.drawPixel(x-r+28, y-r+14, RA8875_BLACK);
  tft.drawPixel(x-r+31, y-r+14, RA8875_BLACK);
  tft.drawPixel(x-r+1, y-r+15, RA8875_BLACK);
  tft.drawPixel(x-r+2, y-r+15, RA8875_BLACK);
  tft.drawPixel(x-r+5, y-r+15, RA8875_BLACK);
  tft.drawPixel(x-r+7, y-r+15, RA8875_BLACK);
  tft.drawPixel(x-r+9, y-r+15, RA8875_BLACK);
  tft.drawPixel(x-r+11, y-r+15, RA8875_BLACK);
  tft.drawPixel(x-r+13, y-r+15, RA8875_BLACK);
  tft.drawPixel(x-r+15, y-r+15, RA8875_BLACK);
  tft.drawPixel(x-r+29, y-r+15, RA8875_BLACK);
  tft.drawPixel(x-r+3, y-r+16, RA8875_BLACK);
  tft.drawPixel(x-r+5, y-r+16, RA8875_BLACK);
  tft.drawPixel(x-r+7, y-r+16, RA8875_BLACK);
  tft.drawPixel(x-r+8, y-r+16, RA8875_BLACK);
  tft.drawPixel(x-r+10, y-r+16, RA8875_BLACK);
  tft.drawPixel(x-r+14, y-r+16, RA8875_BLACK);
  tft.drawPixel(x-r+16, y-r+16, RA8875_BLACK);
  tft.drawPixel(x-r+1, y-r+17, RA8875_BLACK);
  tft.drawPixel(x-r+2, y-r+17, RA8875_BLACK);
  tft.drawPixel(x-r+4, y-r+17, RA8875_BLACK);
  tft.drawPixel(x-r+6, y-r+17, RA8875_BLACK);
  tft.drawPixel(x-r+8, y-r+17, RA8875_BLACK);
  tft.drawPixel(x-r+10, y-r+17, RA8875_BLACK);
  tft.drawPixel(x-r+12, y-r+17, RA8875_BLACK);
  tft.drawPixel(x-r+15, y-r+17, RA8875_BLACK);
  tft.drawPixel(x-r+18, y-r+17, RA8875_BLACK);
  tft.drawPixel(x-r+1, y-r+18, RA8875_BLACK);
  tft.drawPixel(x-r+3, y-r+18, RA8875_BLACK);
  tft.drawPixel(x-r+5, y-r+18, RA8875_BLACK);
  tft.drawPixel(x-r+7, y-r+18, RA8875_BLACK);
  tft.drawPixel(x-r+14, y-r+18, RA8875_BLACK);
  tft.drawPixel(x-r+16, y-r+18, RA8875_BLACK);
  tft.drawPixel(x-r+2, y-r+19, RA8875_BLACK);
  tft.drawPixel(x-r+6, y-r+19, RA8875_BLACK);
  tft.drawPixel(x-r+8, y-r+19, RA8875_BLACK);
  tft.drawPixel(x-r+10, y-r+19, RA8875_BLACK);
  tft.drawPixel(x-r+14, y-r+19, RA8875_BLACK);
  tft.drawPixel(x-r+16, y-r+19, RA8875_BLACK);
  tft.drawPixel(x-r+1, y-r+20, RA8875_BLACK);
  tft.drawPixel(x-r+2, y-r+20, RA8875_BLACK);
  tft.drawPixel(x-r+3, y-r+20, RA8875_BLACK);
  tft.drawPixel(x-r+5, y-r+20, RA8875_BLACK);
  tft.drawPixel(x-r+7, y-r+20, RA8875_BLACK);
  tft.drawPixel(x-r+13, y-r+20, RA8875_BLACK);
  tft.drawPixel(x-r+15, y-r+20, RA8875_BLACK);
  tft.drawPixel(x-r+3, y-r+21, RA8875_BLACK);
  tft.drawPixel(x-r+5, y-r+21, RA8875_BLACK);
  tft.drawPixel(x-r+7, y-r+21, RA8875_BLACK);
  tft.drawPixel(x-r+9, y-r+21, RA8875_BLACK);
  tft.drawPixel(x-r+10, y-r+21, RA8875_BLACK);
  tft.drawPixel(x-r+17, y-r+21, RA8875_BLACK);
  tft.drawPixel(x-r+2, y-r+22, RA8875_BLACK);
  tft.drawPixel(x-r+3, y-r+22, RA8875_BLACK);
  tft.drawPixel(x-r+5, y-r+22, RA8875_BLACK);
  tft.drawPixel(x-r+7, y-r+22, RA8875_BLACK);
  tft.drawPixel(x-r+11, y-r+22, RA8875_BLACK);
  tft.drawPixel(x-r+13, y-r+22, RA8875_BLACK);
  tft.drawPixel(x-r+15, y-r+22, RA8875_BLACK);
  tft.drawPixel(x-r+18, y-r+22, RA8875_BLACK);
  tft.drawPixel(x-r+20, y-r+22, RA8875_BLACK);
  tft.drawPixel(x-r+2, y-r+23, RA8875_BLACK);
  tft.drawPixel(x-r+4, y-r+23, RA8875_BLACK);
  tft.drawPixel(x-r+5, y-r+23, RA8875_BLACK);
  tft.drawPixel(x-r+6, y-r+23, RA8875_BLACK);
  tft.drawPixel(x-r+10, y-r+23, RA8875_BLACK);
  tft.drawPixel(x-r+13, y-r+23, RA8875_BLACK);
  tft.drawPixel(x-r+16, y-r+23, RA8875_BLACK);
  tft.drawPixel(x-r+19, y-r+23, RA8875_BLACK);
  tft.drawPixel(x-r+3, y-r+24, RA8875_BLACK);
  tft.drawPixel(x-r+4, y-r+24, RA8875_BLACK);
  tft.drawPixel(x-r+6, y-r+24, RA8875_BLACK);
  tft.drawPixel(x-r+7, y-r+24, RA8875_BLACK);
  tft.drawPixel(x-r+8, y-r+24, RA8875_BLACK);
  tft.drawPixel(x-r+10, y-r+24, RA8875_BLACK);
  tft.drawPixel(x-r+12, y-r+24, RA8875_BLACK);
  tft.drawPixel(x-r+14, y-r+24, RA8875_BLACK);
  tft.drawPixel(x-r+15, y-r+24, RA8875_BLACK);
  tft.drawPixel(x-r+17, y-r+24, RA8875_BLACK);
  tft.drawPixel(x-r+19, y-r+24, RA8875_BLACK);
  tft.drawPixel(x-r+3, y-r+25, RA8875_BLACK);
  tft.drawPixel(x-r+5, y-r+25, RA8875_BLACK);
  tft.drawPixel(x-r+7, y-r+25, RA8875_BLACK);
  tft.drawPixel(x-r+9, y-r+25, RA8875_BLACK);
  tft.drawPixel(x-r+11, y-r+25, RA8875_BLACK);
  tft.drawPixel(x-r+15, y-r+25, RA8875_BLACK);
  tft.drawPixel(x-r+18, y-r+25, RA8875_BLACK);
  tft.drawPixel(x-r+20, y-r+25, RA8875_BLACK);
  tft.drawPixel(x-r+21, y-r+25, RA8875_BLACK);
  tft.drawPixel(x-r+4, y-r+26, RA8875_BLACK);
  tft.drawPixel(x-r+7, y-r+26, RA8875_BLACK);
  tft.drawPixel(x-r+9, y-r+26, RA8875_BLACK);
  tft.drawPixel(x-r+10, y-r+26, RA8875_BLACK);
  tft.drawPixel(x-r+11, y-r+26, RA8875_BLACK);
  tft.drawPixel(x-r+12, y-r+26, RA8875_BLACK);
  tft.drawPixel(x-r+13, y-r+26, RA8875_BLACK);
  tft.drawPixel(x-r+14, y-r+26, RA8875_BLACK);
  tft.drawPixel(x-r+17, y-r+26, RA8875_BLACK);
  tft.drawPixel(x-r+19, y-r+26, RA8875_BLACK);
  tft.drawPixel(x-r+6, y-r+27, RA8875_BLACK);
  tft.drawPixel(x-r+7, y-r+27, RA8875_BLACK);
  tft.drawPixel(x-r+9, y-r+27, RA8875_BLACK);
  tft.drawPixel(x-r+11, y-r+27, RA8875_BLACK);
  tft.drawPixel(x-r+14, y-r+27, RA8875_BLACK);
  tft.drawPixel(x-r+16, y-r+27, RA8875_BLACK);
  tft.drawPixel(x-r+20, y-r+27, RA8875_BLACK);
  tft.drawPixel(x-r+7, y-r+28, RA8875_BLACK);
  tft.drawPixel(x-r+8, y-r+28, RA8875_BLACK);
  tft.drawPixel(x-r+10, y-r+28, RA8875_BLACK);
  tft.drawPixel(x-r+14, y-r+28, RA8875_BLACK);
  tft.drawPixel(x-r+18, y-r+28, RA8875_BLACK);
  tft.drawPixel(x-r+9, y-r+29, RA8875_BLACK);
  tft.drawPixel(x-r+11, y-r+29, RA8875_BLACK);
  tft.drawPixel(x-r+14, y-r+29, RA8875_BLACK);
  tft.drawPixel(x-r+15, y-r+29, RA8875_BLACK);
  tft.drawPixel(x-r+16, y-r+29, RA8875_BLACK);
  tft.drawPixel(x-r+15, y-r+30, RA8875_BLACK);
  tft.drawPixel(x-r+19, y-r+30, RA8875_BLACK);
  tft.drawPixel(x-r+17, y-r+31, RA8875_BLACK);

}

void iconHouse(Adafruit_RA8875& tft, String timeofday)
{
  // house
  tft.drawRect(22, 280, 20, 12, RA8875_BLACK);

  // door
  tft.drawRect(26, 284, 5, 8, RA8875_BLACK);
  // door handle
  tft.drawLine(26, 287, 28, 287, RA8875_BLACK);

  // window
  if( timeofday == "night")
    tft.fillRect(33, 284, 7, 5, RA8875_RED); // Window light
  tft.drawRect(33, 284, 7, 5, RA8875_BLACK);
  tft.drawLine(36, 284, 36, 288, RA8875_BLACK);
  
  // roof
  tft.drawLine(20, 281, 32, 272, RA8875_BLACK);
  tft.drawLine(32, 272, 43, 281, RA8875_BLACK);

  tft.drawLine(24, 280, 33, 273.5, RA8875_BLACK);
  tft.drawLine(26.5, 280, 33.5, 275, RA8875_BLACK);
  tft.drawLine(29, 280, 35, 275.5, RA8875_BLACK);

  // path and street
  tft.drawLine(26, 292, 30, 296, RA8875_BLACK);
  tft.drawLine(31, 292, 35, 296, RA8875_BLACK);

  tft.drawLine(20, 296, 30, 296, RA8875_BLACK);
  tft.drawLine(35, 296, 48, 296, RA8875_BLACK);

  // street light
  if( timeofday == "night")
  {
    tft.fillCircle(46, 285, 2, RA8875_RED); // Street light
    tft.fillTriangle(46, 283, 42, 294, 50, 294, RA8875_RED); // Street light
  }
  tft.drawLine(46, 286, 46, 294, RA8875_BLACK);
  tft.drawCircle(46, 285, 2, RA8875_BLACK);
  tft.fillRect(45, 284, 3, 1, RA8875_BLACK);
  tft.fillRect(45, 292, 3, 3, RA8875_BLACK);

  // mailbox
  tft.fillRect(23, 293, 3, 1, RA8875_BLACK);
  tft.drawLine(24, 293, 24, 294, RA8875_BLACK);
}

void iconBatteryEmpty( Adafruit_RA8875& tft)
{
  tft.drawRect(5, 5, 12, 7, RA8875_BLACK);
  tft.drawRect(4, 6, 1, 5, RA8875_BLACK);
  tft.fillRect(13, 5, 3, 7, RA8875_BLACK);
}

// Function to draw a circle icon
void iconCircle(Adafruit_RA8875& tft, uint16_t x, uint16_t y, uint16_t r) {
    tft.fillCircle(x, y, r, RA8875_WHITE);
}

// Function to draw a square icon
void iconSquare(Adafruit_RA8875& tft, uint16_t x, uint16_t y, uint16_t size) {
    tft.fillRect(x, y, size, size, RA8875_WHITE);
}

// Function to draw a triangle icon
void iconTriangle(Adafruit_RA8875& tft, uint16_t x, uint16_t y, uint16_t size) {
    tft.fillTriangle(x, y - size, x + size, y + size, x - size, y + size, RA8875_WHITE);
}

void iconButton(Adafruit_RA8875& tft, uint16_t x, uint16_t y, uint16_t w, uint16_t h){
  tft.drawRoundRect(x-5, y-5, w+10, h+10, 30, RA8875_BLACK);
  tft.fillRoundRect(x-5, y-5, w+10, h+10, 30, RA8875_BLACK);
  tft.drawRoundRect(x, y, w, h, 25, LIGHTER_BROWN);
  tft.fillRoundRect(x, y, w, h, 25, LIGHTER_BROWN); 
  tft.drawRoundRect(x+7, y+7, w-14, h-14, 20, LIGHT_BROWN);
  tft.fillRoundRect(x+7, y+7, w-14, h-14, 20, LIGHT_BROWN); 
  tft.fillRoundRect(x+18, y+18, 4, 15, 1, LIGHTER_BROWN);
  tft.fillRoundRect(x+18, y+18, 15, 4, 1, LIGHTER_BROWN);
  tft.fillRoundRect(x+18, y+42, 4, 4, 1, LIGHTER_BROWN);
}

void iconPressButton(Adafruit_RA8875& tft, uint16_t x, uint16_t y, uint16_t w, uint16_t h){
  tft.drawRoundRect(x-5, y-5, w+10, h+10, 30, RA8875_BLACK);
  tft.fillRoundRect(x-5, y-5, w+10, h+10, 30, RA8875_BLACK);
  tft.drawRoundRect(x, y, w, h, 25, LIGHT_BROWN);
  tft.fillRoundRect(x, y, w, h, 25, LIGHT_BROWN); 
  tft.drawRoundRect(x+7, y+7, w-14, h-14, 20, BROWN);
  tft.fillRoundRect(x+18, y+18, 4, 15, 1, LIGHT_BROWN);
  tft.fillRoundRect(x+18, y+18, 15, 4, 1, LIGHT_BROWN);
  tft.fillRoundRect(x+18, y+42, 4, 4, 1, LIGHT_BROWN);
}

// Function to draw a star icon
void iconStar(Adafruit_RA8875& tft, uint16_t x, uint16_t y, uint16_t size) {
    // Star drawing requires more complex trigonometry, this is a basic example
    const int points = 5 * 2;
    double angle = PI / 5.0;
    uint16_t px[points], py[points];
    for (int i = 0; i < points; i++) {
        double r = size * (i % 2 == 0 ? 1.0 : 0.5);
        px[i] = x + r * sin(i * angle);
        py[i] = y + r * cos(i * angle);
    }
    for (int i = 0; i < points; i++) {
        tft.drawLine(px[i], py[i], px[(i + 1) % points], py[(i + 1) % points], RA8875_WHITE);
    }
}

// Function to draw a diamond icon
void iconDiamond(Adafruit_RA8875& tft, uint16_t x, uint16_t y, uint16_t size) {
    tft.fillTriangle(x, y - size, x + size, y, x, y + size, RA8875_WHITE);
    tft.fillTriangle(x, y + size, x - size, y, x, y - size, RA8875_WHITE);
}

// Function to draw a heart icon
void iconHeart(Adafruit_RA8875& tft, uint16_t x, uint16_t y, uint16_t size) {
    uint16_t width = size, height = size;
    tft.fillCircle(x - width/4, y - height/4, width/4, RA8875_WHITE);
    tft.fillCircle(x + width/4, y - height/4, width/4, RA8875_WHITE);
    tft.fillTriangle(x - width/2, y - height/4, x + width/2, y - height/4, x, y + height/2, RA8875_WHITE);
}

// Function to draw a heart icon with outline
void iconPixelHeart(Adafruit_RA8875& tft, uint16_t x, uint16_t y, uint16_t size) {
    uint16_t width = size, height = size;
    tft.fillCircle(x - width/4, y - height/4, width/4, RA8875_BLACK);
    tft.fillCircle(x + width/4, y - height/4, width/4, RA8875_BLACK);
    tft.fillTriangle(x - width/2, y - height/4, x + width/2, y - height/4, x, y + height/2, RA8875_BLACK);

    tft.fillCircle(x - width/4, y - height/4, width/4, RED);
    tft.fillCircle(x + width/4, y - height/4, width/4, RED);
    tft.fillTriangle(x - width/2, y - height/4, x + width/2, y - height/4, x, y + height/2, RED);
}

void iconPixelatedTree1(Adafruit_RA8875& tft, uint16_t x, uint16_t y, uint16_t pixelSize) {
  uint16_t trunkColor = 0x89A4;
  uint16_t leafColor = RA8875_GREEN;
  
  // Define the pattern of the tree, 1 for leaves, 2 for trunk, 0 for empty space
  const int patternHeight = 7;  // Adjust the size to fit your image
  const int patternWidth = 6;   // Adjust the size to fit your image
  int pattern[patternHeight][patternWidth] = {
    {1, 1, 1, 0, 0, 0},
    {0, 1, 1, 1, 0, 1},
    {0, 0, 1, 1, 1, 1},
    {0, 1, 1, 1, 1, 0},
    {0, 1, 2, 1, 0, 0},
    {0, 0, 2, 0, 0, 0},
    {0, 0, 2, 0, 0, 0}
  };

  // Draw the tree based on the pattern
  for (int i = 0; i < patternHeight; i++) {
    for (int j = 0; j < patternWidth; j++) {
      uint16_t color;
      if (pattern[i][j] == 1) {
        color = leafColor;
      } else if (pattern[i][j] == 2) {
        color = trunkColor;
      } else {
        continue; // Skip if the space should be empty
      }
      
      // Calculate the top-left corner of the current "pixel"
      uint16_t currentX = x + j * pixelSize;
      uint16_t currentY = y + i * pixelSize;
      
      // Draw the "pixel"
      tft.fillRect(currentX, currentY, pixelSize, pixelSize, color);
    }
  }
}

void iconPixelatedTree2(Adafruit_RA8875& tft, uint16_t x, uint16_t y, uint16_t pixelSize) {
  uint16_t trunkColor = 0x89A4;
  uint16_t leafColor = RA8875_GREEN;
  
  // Define the pattern of the tree, 1 for leaves, 2 for trunk, 0 for empty space
  const int patternHeight = 7;  // Adjust the size to fit your image
  const int patternWidth = 6;   // Adjust the size to fit your image
  int pattern[patternHeight][patternWidth] = {
    {0, 0, 1, 1, 0, 0},
    {0, 1, 1, 1, 1, 0},
    {1, 1, 1, 1, 1, 1},
    {0, 1, 1, 1, 1, 0},
    {0, 1, 2, 1, 0, 0},
    {0, 0, 2, 0, 0, 0},
    {0, 0, 2, 0, 0, 0}
  };

  // Draw the tree based on the pattern
  for (int i = 0; i < patternHeight; i++) {
    for (int j = 0; j < patternWidth; j++) {
      uint16_t color;
      if (pattern[i][j] == 1) {
        color = leafColor;
      } else if (pattern[i][j] == 2) {
        color = trunkColor;
      } else {
        continue; // Skip if the space should be empty
      }
      
      // Calculate the top-left corner of the current "pixel"
      uint16_t currentX = x + j * pixelSize;
      uint16_t currentY = y + i * pixelSize;
      
      // Draw the "pixel"
      tft.fillRect(currentX, currentY, pixelSize, pixelSize, color);
    }
  }
}

void iconPixelatedTree3(Adafruit_RA8875& tft, uint16_t x, uint16_t y, uint16_t pixelSize) {
  uint16_t trunkColor = 0x89A4;
  uint16_t leafColor = RA8875_GREEN;
  
  // Define the pattern of the tree, 1 for leaves, 2 for trunk, 0 for empty space
  const int patternHeight = 8;  // Adjust the size to fit your image
  const int patternWidth = 6;   // Adjust the size to fit your image
  int pattern[patternHeight][patternWidth] = {
    {0, 0, 1, 1, 0, 0},
    {1, 0, 1, 1, 0, 0},
    {1, 1, 1, 1, 1, 0},
    {0, 1, 1, 1, 1, 0},
    {0, 0, 1, 1, 1, 0},
    {0, 0, 1, 1, 1, 0},
    {0, 0, 2, 2, 0, 0},
    {0, 0, 2, 2, 0, 0}
  };

  // Draw the tree based on the pattern
  for (int i = 0; i < patternHeight; i++) {
    for (int j = 0; j < patternWidth; j++) {
      uint16_t color;
      if (pattern[i][j] == 1) {
        color = leafColor;
      } else if (pattern[i][j] == 2) {
        color = trunkColor;
      } else {
        continue; // Skip if the space should be empty
      }
      
      // Calculate the top-left corner of the current "pixel"
      uint16_t currentX = x + j * pixelSize;
      uint16_t currentY = y + i * pixelSize;
      
      // Draw the "pixel"
      tft.fillRect(currentX, currentY, pixelSize, pixelSize, color);
    }
  }
}

void iconPopBox(Adafruit_RA8875& tft, uint16_t x, uint16_t y, uint16_t w, uint16_t h, uint16_t r){
  tft.fillRoundRect(x, y, w, h, 25, RA8875_BLACK);
  tft.fillRoundRect(x + 5, y + 5, w - 10, h - 10, 20, LIGHTER_BROWN);
  tft.fillRoundRect(x + 12, y + 12, w - 24, h - 24, 15, LIGHT_BROWN);
}

void iconWindow(Adafruit_RA8875& tft, uint16_t x, uint16_t y, uint16_t w, uint16_t h, uint16_t r){
  tft.fillRoundRect(x, y, w, h, r, RA8875_BLACK);
  tft.fillRoundRect(x + 5, y + 5, w - 10, h - 10, r, BROWN);
  tft.fillRect(x + 5, y + 30, w - 10, 30, LIGHTER_BROWN);
  tft.fillRoundRect(x + 5, y + 30, w - 10, h - 35, r - 5, LIGHTER_BROWN);
  tft.fillRect(x + 9, y + 34, w - 18, 30, LIGHT_BROWN);
  tft.fillRoundRect(x + 9, y + 34, w - 18, h - 43, r - 5, LIGHT_BROWN);
  tft.fillCircle(x + 25, y + 16, 4, RA8875_BLACK);
  tft.fillCircle(x + 45, y + 16, 4, RA8875_BLACK);
  tft.fillCircle(x + 65, y + 16, 4, RA8875_BLACK);
  tft.fillCircle(x + 25, y + 16, 3, GREEN);
  tft.fillCircle(x + 45, y + 16, 3, YELLOW);
  tft.fillCircle(x + 65, y + 16, 3, RED);
}

void iconMushroom(Adafruit_RA8875& tft, uint16_t x, uint16_t y, uint16_t w, uint16_t h, uint16_t r){
  tft.fillRoundRect(x, y, w, h, r, RA8875_BLACK);  // outline
  tft.fillRoundRect(x, y - 30, w, h, 2*r, RA8875_BLACK); 
  tft.fillRoundRect(x + (x/5), y+h, w - (2*(x/5)), 0.9*h, 0.7*r, RA8875_BLACK); 
  tft.fillCircle(x + (x/4), y + h, x/4, RA8875_BLACK);
  tft.fillCircle(x + (x*0.75), y + h, x/4, RA8875_BLACK);

  // Color
  // tft.fillRoundRect(x, y - 30, w, h, 20 + offset, RA8875_BLACK);
  // tft.fillRoundRect(x + 3, y + 3, w - 6, h - 6 , 20, DARK_RED);  // shadow
  // tft.fillRoundRect(x + 3, y + 3, w - 6, h - 2 , 20, RED);  // red cap
  // tft.fillRoundRect(x + 3, y + 3 - 30, w -6, h - 6, 20 + offset, RED); // red cap
  // tft.fillCircle(x,y,r, RA8875_WHITE); // make lots of white circles

  // tft.fillRoundRect(x + 3 + (x/5), y+h+3, w - (2*(x/5)) - 6, 0.9*h - 6, 25, LIGHT_BROWN); 
  // tft.fillRoundRect(x + 3 + (x/5), y+h+6, w - (2*(x/5)) - 6, 0.9*h - 9, 25, LIGHTER_BROWN); 
  // fill two circles for outlines (radius is 0.9 *r)
  // eyes (two black rectangles) + (two white rectangles inside)
  // add blush (two rectangles that are pink)
}


// Note: Customize the function parameters and drawing details according to the actual requirements and dimensions needed.
