#include <Arduino.h>
#include <Adafruit_GFX.h>    
#include <SPI.h>
#include <Wire.h>
#include "Adafruit_RA8875.h"
#include "SPIFFS.h"
#include <FS.h>
#include "TJpg_Decoder.h"
#include "ESP32Time.h"
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/queue.h"
#include <driver/i2s.h>
#include "sos-iir-filter.h"
#include <stdint.h>
#include <WiFi.h>
#include "owm_credentials.h"
#include <HTTPClient.h>
#include <ArduinoJson.h>
#include "time.h"
#include <GxEPD2_BW.h>
#include <GxEPD2_3C.h>
#include <U8g2_for_Adafruit_GFX.h>
#include "lang.h"
#include <BluetoothA2DPSink.h>
#include <WebServer.h>
#include "icons.h"
#include "XT_DAC_Audio.h"
#include "TestAudio.h"

/// COLORS ///
#define BROWN 0x8244
#define RED 0xF082
#define GREEN 0x4DA2
#define OFF_WHITE 0xEF5D
#define GRAY 0xAD34
#define LIGHT_BROWN 0xCCAC
#define LIGHTER_BROWN 0xEDD0
#define DARKER_BROWN 0x72E8
#define LIGHT_GREEN 0x8E25

bool received = false;

/// RTOS TASKS ///

// RTOS Task Configurations
#define configUSE_PREEMPTION 1
#define INCLUDE_vTaskSuspend 1
#define configUSE_TIME_SLICING 1
#define INCLUDE_vTaskDelayUntil 1
#define INCLUDE_vTaskDelay 1
#define configUSE_TIMERS 1
#define configTIMER_TASK_PRIORITY 3
#define configTIMER_TASK_STACK_DEPTH (configMINIMAL_STACK_SIZE * 4)

// Task prototypes
void ButtonTask(void *parameter);                   // button handling task 
void DisplayTask(void *parameter);                  // update display mode task

// Define Queue Handles
QueueHandle_t displayQueue;                         // display button queue
QueueHandle_t buttonQueue;                          // button press queue
QueueHandle_t timerQueue;                           // timer active queue
QueueHandle_t bluetoothQueue;                       // bluetooth queue
QueueHandle_t clockQueue;                           // update clock (home display) queue

// Timer Interrupts & Handles
TimerHandle_t updateclock;
TimerHandle_t updateTimer;
void updateClock(TimerHandle_t xTimer);             // update current time 
void updateTimerDisplay(TimerHandle_t xTimer);      // update active timer 

///******************** BUTTON HANDLING *******************///

// Definitions
#define sw1 17
#define sw2 16
#define sw3 4
#define sw4 0
#define sw5 32
#define sw6 15
#define NUM_BUTTONS 6

// Global variables
int trackButton;
int lastDebounce1Time = 0;
int lastDebounce2Time = 0; 
int lastDebounce3Time = 0; 
int lastDebounce4Time = 0; 
int lastDebounce5Time = 0; 
int lastDebounce6Time = 0; 
volatile int buttons_pressed;

// Button Task Interrupts
void IRAM_ATTR sw2_interrupt();
void IRAM_ATTR sw3_interrupt();
void IRAM_ATTR sw4_interrupt();
void IRAM_ATTR sw5_interrupt();
void IRAM_ATTR sw6_interrupt();

// Function Declarations
void button2(int currdisp);
void button3(int currdisp);
void button4(int currdisp);
void button5(int currdisp);

///*************************** DISPLAY MODE *************************///

// Global variables 
int displaycounter = 1;                         // counter for display case
int activeDisplayMode = 1;                      // variable keeping track of display mode

// Function declarations //
void updateDefaultDisplay(int displayMode);     // update display mode
void HomeDisplay();                             // home display (1)
void RemindersDisplay();                        // display reminders (2)
void NightLightDisplay();                       // night light (3)
void MusicDisplay();                            // sleep display (4)
void TimerDisplay();                            // timer display (5)

// ESP32 RTC
ESP32Time rtc(3600);

///************** BLUETOOTH ***************///

// Global Variables & Declarations
BluetoothA2DPSink a2dp_sink;
TaskHandle_t Task0;
TaskHandle_t Task1;
bool bluetoothActive = false;
bool bluetoothPlay = false;
int currentVolume;
XT_DAC_Audio_Class DacAudio(25,0); 
XT_Wav_Class Alarm (TestAudio_wav);

// Bluetooth Config Settings
const i2s_config_t i2s_config2 = {
  .mode = (i2s_mode_t)(I2S_MODE_MASTER | I2S_MODE_TX | I2S_MODE_DAC_BUILT_IN),
  .sample_rate = 44100,
  .bits_per_sample = (i2s_bits_per_sample_t)16,
  .channel_format = I2S_CHANNEL_FMT_RIGHT_LEFT,
  .communication_format = (i2s_comm_format_t)I2S_COMM_FORMAT_STAND_MSB,
  .intr_alloc_flags = 0,
  .dma_buf_count = 8,
  .dma_buf_len = 64,
  .use_apll = false
};

const i2s_pin_config_t pin_config2 = {
  bck_io_num:   -1,  
  ws_io_num:    -1,    
  data_out_num: 25, // not used
  data_in_num:  -1   
};

// Function Declarations
void handleBluetoothCommands(void* parameter); 

///****************** TIMER STUFF *************************///

// Global Variables 
unsigned long previousMillis = 0;                   // variable to store the previous millis() value
unsigned long interval = 1000;                      // interval for updating the timer display (1 second)
int user_min = 1;                                   // user defined time
int remainingSeconds = user_min * 60;               // total time remaining
int minutes = remainingSeconds / 60;                // convert total time to minutes
int seconds = remainingSeconds % 60;                // convert total time to seconds
bool timerActive = false;                           // timer active flag
int screenlight = 0;

// Function Declarations 
void startTimer(int &remainingSeconds, int &minutes, int &seconds);
void stopTimer(int &remainingSeconds, int &minutes, int &seconds);
void timerDone(int &remainingSeconds,int &minutes, int &seconds);
void increaseTimer();
void decreaseTimer();
void timerReset();

///********************** WIFI ********************///

// Global Variables 
String serverPath = "http://api.openweathermap.org/data/2.5/weather?lat=41.676388&lon=-86.250275&appid=821c321eeb2d9139b7a241c18c03280e";
const char* owmApiKey = "821c321eeb2d9139b7a241c18c03280e";
const char* ssid = "SDNet";
const char* password = "CapstoneProject";
const char* cityId = "4926563"; // Find your city ID at https://openweathermap.org/find
const char* units = "imperial"; // Use "imperial" for Fahrenheit/"metric" for Celsius.
String jsonBuffer;
struct tm timeinfo;
int timeCheck;
String currentWeather;
String currentIcon;
int currentTemp;

WebServer server(80);

// South Bend Time Zone
const char* ntpServer = "pool.ntp.org";
const long  gmtOffset_sec = -18000;
const int   daylightOffset_sec = 0;

// Function Declarations 
void connectToWiFi();
bool checkTimeinfo();
void updateWeatherCondition(const String& newCondition, const String& newIcon);
void drawWeather();
String httpGETRequest(const char* serverName);
void weatherAPI();

void handleRoot();
void handleUpload();

///************************ FLASH & SPI *****************************///

// Global Variables & Definitions 

// CS and RST assignments 
#define RA8875_CS 5
#define RA8875_RESET 33
Adafruit_RA8875 tft = Adafruit_RA8875(RA8875_CS, RA8875_RESET);

// Flash Specific Pins
#define FLASH_CS   13  // Chip Select
#define FLASH_HOLD 21 // HOLD
#define FLASH_WP   22 // Write Protect

// SPI Bus Settings 
SPISettings spiSettings(4000000, MSBFIRST, SPI_MODE0); // 4MHz SPI clock, might need adjustment

// Image File Address 
uint32_t sunny_address = 0x100000;  
// FLASH IMAGE ADDRESSES //
// Square images for home display
uint32_t EELOGO = 0x000000;

uint32_t MUSIC = 0x0E6000;

// Functions Declarations
void initFlash() {
    pinMode(FLASH_HOLD, OUTPUT);
    pinMode(FLASH_WP, OUTPUT);
    digitalWrite(FLASH_HOLD, HIGH); // Disable HOLD
    digitalWrite(FLASH_WP, HIGH);   // Disable write protect
    
    pinMode(FLASH_CS, OUTPUT);
    digitalWrite(FLASH_CS, HIGH); // Deselect the flash chip
    // Additional initialization for flash chip if necessary
}

void readFlashData(uint32_t address, uint8_t* buffer, size_t length) {
    digitalWrite(FLASH_CS, LOW);
    SPI.beginTransaction(spiSettings);
    SPI.transfer(0x03); // Read command
    SPI.transfer((address >> 16) & 0xFF);
    SPI.transfer((address >> 8) & 0xFF);
    SPI.transfer(address & 0xFF);
    for (size_t i = 0; i < length; i++) {
        buffer[i] = SPI.transfer(0x00); // Dummy byte to clock out the data
    }
    SPI.endTransaction();
    digitalWrite(FLASH_CS, HIGH);
}

// BMP file header and info header structures
struct BMPFileHeader {
    uint16_t file_type{0x4D42}; // File type always BM which is 0x4D42
    uint32_t file_size{0};      // Size of the file (in bytes)
    uint16_t reserved1{0};      // Reserved, always 0
    uint16_t reserved2{0};      // Reserved, always 0
    uint32_t offset_data{0};    // Start position of pixel data
};

struct BMPInfoHeader {
    uint32_t size{0};           // Size of this header (in bytes)
    int32_t width{0};           // width of bitmap in pixels
    int32_t height{0};          // height of bitmap in pixels
    uint16_t planes{1};         // No. of planes for the target device, this is always 1
    uint16_t bit_count{0};      // No. of bits per pixel
    uint32_t compression{0};    // Type of compression
    uint32_t size_image{0};     // Size of pixel data
    int32_t x_pixels_per_meter{0};
    int32_t y_pixels_per_meter{0};
    uint32_t colors_used{0};    // No. of colors used by the bitmap
    uint32_t colors_important{0}; // No. of colors that are important
};

// Function to read BMP headers
bool readBMPHeaders(File &file, BMPFileHeader &fileHeader, BMPInfoHeader &infoHeader) {
    if (file.read((uint8_t *)&fileHeader, sizeof(fileHeader)) != sizeof(fileHeader)) {
        return false;
    }
    if (file.read((uint8_t *)&infoHeader, sizeof(infoHeader)) != sizeof(infoHeader)) {
        return false;
    }
    return true;
}

// Function to convert RGB888 to RGB565
uint16_t rgb888ToRgb565(uint8_t red, uint8_t green, uint8_t blue) {
    return ((red >> 3) << 11) | ((green >> 2) << 5) | (blue >> 3);
}

// Function to draw BMP file
void drawBMP(uint32_t bmpAddress, uint16_t x_start, uint16_t y_start) {
    // Buffer for reading the BMP header
    uint8_t bmpHeader[54]; // Standard BMP header size
    readFlashData(bmpAddress, bmpHeader, sizeof(bmpHeader));

    // Extract relevant fields from the header
    uint32_t pixelDataOffset = *(uint32_t *)&bmpHeader[10];
    int32_t width = *(int32_t *)&bmpHeader[18];
    int32_t height = *(int32_t *)&bmpHeader[22];
    uint16_t bitDepth = *(uint16_t *)&bmpHeader[28];

    if (bitDepth != 24) {
        Serial.println("BMP is not 24-bit depth");
        return;
    }

    // Calculate the size of one row of raw pixel data
    uint32_t rowSize = width * 3; // 3 bytes per pixel for RGB888
    uint8_t *rawRow = new uint8_t[rowSize]; // Allocate memory for one row

    // Allocate memory for the RGB565 pixel array for one row
    uint16_t *pixelRow = new uint16_t[width];

    // Determine if the BMP image is stored top-down or bottom-up
    bool topDown = height < 0;
    height = abs(height);

    // Adjust starting Y coordinate based on image orientation
    uint16_t y_offset = topDown ? y_start : y_start + height - 1;

    for (int y = 0; y < height; y++) {
        // Read one row of raw pixel data from the flash
        uint32_t rowAddress = bmpAddress + pixelDataOffset + (topDown ? y * rowSize : (height - 1 - y) * rowSize);
        readFlashData(rowAddress, rawRow, rowSize);

        // Convert to RGB565 and store in pixelRow
        for (int x = 0; x < width; x++) {
            int idx = x * 3;
            uint8_t blue = rawRow[idx];
            uint8_t green = rawRow[idx + 1];
            uint8_t red = rawRow[idx + 2];
            pixelRow[x] = rgb888ToRgb565(red, green, blue);
        }

        // Draw the row on the screen
        tft.drawPixels(pixelRow, width, x_start, topDown ? y_offset + y : y_offset - y);
    }

    // Clean up
    delete[] rawRow;
    delete[] pixelRow;
}
int button_press;

// Setup
void setup(){
  Serial.begin(115200);
  initFlash();                          // Initialize flash
  delay(1000);                          // Safety

  ///******** RTOS TASK MANAGEMENT & INITIALIZATIONS ***********///

  // Create queues
  displayQueue = xQueueCreate(1, sizeof(int));
  buttonQueue = xQueueCreate(1, sizeof(int));
  bluetoothQueue = xQueueCreate(1, sizeof(bool));
  clockQueue = xQueueCreate(1, sizeof(int));

  // Safety check for queues
  if (displayQueue == NULL) {
    Serial.println("Queue create failed");
    while(1); // Infinite loop to halt operation
  }

  // Attach interrupts
  attachInterrupt(digitalPinToInterrupt(sw2), sw2_interrupt, FALLING);
  attachInterrupt(digitalPinToInterrupt(sw3), sw3_interrupt, FALLING);
  attachInterrupt(digitalPinToInterrupt(sw4), sw4_interrupt, FALLING);
  attachInterrupt(digitalPinToInterrupt(sw5), sw5_interrupt, FALLING);
  attachInterrupt(digitalPinToInterrupt(sw6), sw6_interrupt, FALLING);

  // Create task for button handling
  xTaskCreatePinnedToCore(
    ButtonTask,               // Task function
    "Button Task",            // Name of the task
    5000,                     // Stack size in words
    NULL,                     // Task parameters
    4,                        // Task priority
    NULL,                     // Task Handle
    1                         // Pinned to core 0 (CoreID)
  );           

  // Create task for bluetooth 
  xTaskCreatePinnedToCore(handleBluetoothCommands, "handleBluetoothCommands", 2500, NULL, 1, &Task1, 0);

  // Create task for updating display mode
  xTaskCreatePinnedToCore(DisplayTask, "Display", 4000, NULL, 4, NULL, 1);

  // Create timer interrupts
  updateclock = xTimerCreate("Clock Update", pdMS_TO_TICKS(1000), pdFALSE, ( void * ) 0, updateClock);
  updateTimer = xTimerCreate("Start Timer", pdMS_TO_TICKS(1000), pdFALSE, ( void * )0, updateTimerDisplay);

  ///*********** SPI & FLASH CONFIG **********///

  SPI.begin();

  // Safety checks for SPI
  if (!SPIFFS.begin(true)) {
    Serial.println("An Error has occurred while mounting SPIFFS");
    while(1) yield();
  }

  if (!tft.begin(RA8875_800x480)) {
    Serial.println("RA8875 Not Found!");
    while (1);
  }

  // tft initializations & config
  tft.setRotation(0);
  tft.displayOn(true);
  tft.GPIOX(true); // Enable TFT - display enable tied to GPIOX
  tft.PWM1config(true, RA8875_PWM_CLK_DIV1024); // PWM output for backlight
  tft.PWM1out(255);
  tft.fillScreen(RA8875_BLACK);
  tft.graphicsMode();
  tft.cp437(true);

  // Connect to Wifi to set initial time
  connectToWiFi();

  configTime(gmtOffset_sec, daylightOffset_sec, ntpServer);
  getLocalTime(&timeinfo);
  rtc.setTimeStruct(timeinfo);

  // Get real time and weather
  timeCheck = rtc.getMinute();
  weatherAPI();

  // Disconnect WiFi
  WiFi.disconnect();

  ///********* KEYBOARD PIN CONFIG ********///

  pinMode(sw1, INPUT);
  pinMode(sw2, INPUT);
  pinMode(sw3, INPUT);
  pinMode(sw4, INPUT);
  pinMode(sw5, INPUT);

  // Initialize to home screen & start updateClock timer interrupt
  updateDefaultDisplay(1);
  xTimerStart(updateclock, 0);
}

// Not using loop
void loop(){
  // do nothing
}

///***************** CN INTERRUPTS FOR BUTTON PRESSES ***************///


void IRAM_ATTR sw2_interrupt(){
  BaseType_t pxHigherPriorityTaskWoken = pdFALSE;
  unsigned long interrupt2_time = millis(); 
  if(interrupt2_time - lastDebounce2Time > 200){
    lastDebounce2Time = 0; 
    buttons_pressed = sw2;
    xQueueSendFromISR(buttonQueue, &button_press, NULL);
    if (pxHigherPriorityTaskWoken) {
      portYIELD_FROM_ISR();
    }
  }
  lastDebounce2Time = interrupt2_time;
}

void IRAM_ATTR sw3_interrupt(){
  BaseType_t pxHigherPriorityTaskWoken = pdFALSE;
  unsigned long interrupt3_time = millis(); 
  if(interrupt3_time - lastDebounce3Time > 300){
    lastDebounce3Time = 0; 
    buttons_pressed = sw3;
    xQueueSendFromISR(buttonQueue, &button_press, NULL);
    if (pxHigherPriorityTaskWoken) {
      portYIELD_FROM_ISR();
    }
  }
  lastDebounce3Time = interrupt3_time;
}

void IRAM_ATTR sw4_interrupt(){
  BaseType_t pxHigherPriorityTaskWoken = pdFALSE;
  unsigned long interrupt4_time = millis(); 
  if(interrupt4_time - lastDebounce4Time > 300){
    lastDebounce4Time = 0; 
    buttons_pressed = sw4;
    xQueueSendFromISR(buttonQueue, &button_press, NULL);
    if (pxHigherPriorityTaskWoken) {
      portYIELD_FROM_ISR();
    }
  }
  lastDebounce4Time = interrupt4_time;
}

void IRAM_ATTR sw5_interrupt(){
  BaseType_t pxHigherPriorityTaskWoken = pdFALSE;
  unsigned long interrupt5_time = millis(); 
  if(interrupt5_time - lastDebounce5Time > 300){
    lastDebounce5Time = 0; 
    buttons_pressed = sw5;
    xQueueSendFromISR(buttonQueue, &button_press, NULL);
    if (pxHigherPriorityTaskWoken) {
      portYIELD_FROM_ISR();
    }
  }
  lastDebounce5Time = interrupt5_time;
}

void IRAM_ATTR sw6_interrupt(){
  BaseType_t pxHigherPriorityTaskWoken = pdFALSE; 
  unsigned long interrupt6_time = millis(); 
  if(interrupt6_time - lastDebounce6Time > 200){
    lastDebounce6Time = 0;
    displaycounter++; // Increment counter
    if (displaycounter > 5){
      displaycounter = 1; // Wrap around to the first mode
    }
    xQueueSendFromISR(displayQueue, &displaycounter, NULL);
    if (pxHigherPriorityTaskWoken) {
      portYIELD_FROM_ISR();
    }
  }
  lastDebounce1Time = interrupt6_time;
}

///********************** BUTTON HANDLING *********************///

// Main Button Handler Task
void ButtonTask(void *parameter) {
  int pressed;
  int button;
  // Loop forever
  while (true) {
    if(xQueueReceive(buttonQueue, &pressed, portMAX_DELAY)){
      button = buttons_pressed;
      switch(button){
        case sw2:
          button2(activeDisplayMode);
          break;
        case sw3:
          button3(activeDisplayMode);
          break;
        case sw4:
          button4(activeDisplayMode);
          break;
        case sw5:
          button5(activeDisplayMode);
          break;
        default:
          break;
      }  
    }
    vTaskDelay(50/ portTICK_RATE_MS);
  }
}

// Button 2 Logic
void button2(int currdisp){
  switch(activeDisplayMode){
    case 2:
      iconWindow(tft, 30, 30, 740, 430, 15);
      connectToWiFi();
      tft.textMode();
      tft.setTextSize(1);
      tft.textEnlarge(1);
      tft.textSetCursor(50, 70);
      tft.textTransparent(RA8875_WHITE);
      tft.print("Connecting to WiFi...");
      Serial.print("IP Address: ");
      Serial.println(WiFi.localIP());
      iconWindow(tft, 30, 30, 740, 430, 15);
      tft.textSetCursor(50, 70);
      tft.textTransparent(RA8875_WHITE);
      tft.print("IP Address: ");
      tft.print(WiFi.localIP());
      server.begin();
      server.on("/", HTTP_GET, handleRoot);
      server.on("/upload", HTTP_POST, handleUpload);
      Serial.println("HTTP server started");
      tft.textSetCursor(50, 100);
      tft.textTransparent(RA8875_WHITE);
      tft.print("Waiting for upload...");
      while(!received){
        server.handleClient();
      }
      WiFi.disconnect();
      Serial.println("wifi disconnected");
      received = false;
      break;
    case 3:
      screenlight = !screenlight;
      if(screenlight == 0){
        NightLightDisplay();
      } else {
        tft.fillScreen(RA8875_WHITE);
      }
      break;
    case 4:
      xQueueSend(bluetoothQueue, &bluetoothActive, portMAX_DELAY);
      // button flashing code //
      if (!bluetoothActive) {
        tft.graphicsMode();
        iconPressButton(tft, 288, 245, 220, 100);  // flash the button 
        delay(20);
        iconButton(tft, 288, 245, 220, 100);
        tft.textMode();
        tft.textTransparent(LIGHTER_BROWN);
        tft.setTextSize(1);
        tft.textEnlarge(0);
        tft.textSetCursor(378, 310); tft.print("BT on");
        tft.setTextSize(5);
        tft.textEnlarge(3);
        tft.textSetCursor(386, 250);  // (x + 98) and (y + 5)
        tft.write(0x0E); // Music Symbol
      }
      else{
        tft.graphicsMode();
        iconPressButton(tft, 288, 245, 220, 100); // flash the button 
        delay(20);
        tft.graphicsMode();
        iconButton(tft, 288, 245, 220, 100);
        tft.textMode();
        tft.textTransparent(BROWN);
        tft.setTextSize(1);
        tft.textEnlarge(0);
        tft.textSetCursor(375, 310); tft.print("BT off");
        tft.setTextSize(5);
        tft.textEnlarge(3);
        tft.textSetCursor(386, 250);  // (x + 98) and (y + 5)
        tft.write(0x0E); // Music Symbol
      }
      // end button flashing code //

      break;
    // Start/Stop timer
    case 5:
      if(timerActive){
        xTimerStop(updateTimer, 0);
        timerActive = false;
        stopTimer(remainingSeconds, minutes, seconds);
        /// UPDATING BUTTON COLOR ///
        tft.graphicsMode();
        iconPressButton(tft, 288, 245, 220, 100);  // flash the button 
        delay(20);
        tft.graphicsMode();
        iconButton(tft, 288, 245, 220, 100);
        tft.textMode();
        tft.textTransparent(BROWN);
        tft.setTextSize(1);
        tft.textEnlarge(0);
        tft.textSetCursor(380, 305);
        tft.print("Start");
        tft.setTextSize(5);
        tft.textEnlarge(3);
        tft.textSetCursor(386, 250); 
        tft.write(0x10); 
        /// END UPDATING BUTTON COLOR ///
      } else {
        startTimer(remainingSeconds, minutes, seconds);
        /// UPDATING BUTTON COLOR ///
        tft.graphicsMode();
        iconPressButton(tft, 288, 245, 220, 100);  // flash the button 
        delay(20);
        tft.graphicsMode();
        iconButton(tft, 288, 245, 220, 100);
        tft.textMode();
        tft.textTransparent(LIGHTER_BROWN);
        tft.setTextSize(1);
        tft.textEnlarge(0);
        tft.textSetCursor(382, 305);
        tft.print("Stop");
        tft.setTextSize(5);
        tft.textEnlarge(3);
        tft.textSetCursor(383, 250); 
        tft.write(0x04); // Pause Symbol
        /// END UPDATING BUTTON COLOR ///
      }
      break;
    default:
      break;
  }
}

// Button 3 Logic
void button3(int currdisp){
  switch(activeDisplayMode){
    case 2:
      if(WiFi.status() == WL_CONNECTED){
        WiFi.disconnect();
        received = false;
      }
      iconWindow(tft, 30, 30, 740, 430, 15);
      tft.textMode();
      tft.setTextSize(1);
      tft.textEnlarge(1);
      tft.textSetCursor(40, 40);
      tft.textTransparent(RA8875_WHITE);
      tft.print("Wifi Disconnected");
      Serial.print("Wifi Disconnected");
      break;
    // Pause/Play music
    case 4:
      if(bluetoothActive){
        if(bluetoothPlay){
          bluetoothPlay = false;
          a2dp_sink.pause();
          // button flashing code //
          tft.graphicsMode();
          iconPressButton(tft, 536, 245, 220, 100);    // Flash Button Press
          delay(20);
          iconButton(tft, 536, 245, 220, 100);    
          tft.textMode();
          tft.setTextSize(5);
          tft.textEnlarge(3);
          tft.textTransparent(BROWN);
          tft.textSetCursor(634, 250); 
          tft.write(0x10); // Play Symbol
          tft.setTextSize(1);
          tft.textEnlarge(0);
          tft.textSetCursor(630, 305);  tft.print("Play");
          // end button flashing code
        } else if (!bluetoothPlay){
          bluetoothPlay = true;
          a2dp_sink.play();
          // button flashing code //
          tft.graphicsMode();
          iconPressButton(tft, 536, 245, 220, 100);    // Flash Button Press
          delay(20);
          iconButton(tft, 536, 245, 220, 100);    
          tft.textMode();
          tft.setTextSize(5);
          tft.textEnlarge(3);
          tft.textTransparent(LIGHTER_BROWN);
          tft.textSetCursor(634, 250); 
          tft.write(0x04); // Pause Symbol
          tft.setTextSize(1);
          tft.textEnlarge(0);
          tft.textSetCursor(630, 305);  tft.print("Pause");
          // end button flashing code
        }
      }
      break;
    // Reset Timer
    case 5:
        xTimerStop(updateTimer, 0);
        timerActive = false;
        timerReset();
      // button flashing code //
        tft.graphicsMode();
        iconPressButton(tft, 536, 245, 220, 100);    // Flash Button Press
        delay(20);
        iconButton(tft, 536, 245, 220, 100);    
        tft.textMode();
        tft.setTextSize(5);
        tft.textEnlarge(3);
        tft.textTransparent(BROWN);
        tft.textSetCursor(634, 250); 
        tft.write(0x2A); // Reset
        tft.setTextSize(1);
        tft.textEnlarge(0);
        tft.textSetCursor(620, 305);  tft.print("Reset Timer");
        // end button flashing code //
      break;
    default:
      break;
  }
}

// Button 4 Logic
void button4(int currdisp){
  switch(activeDisplayMode){
    case 2:
      if(WiFi.status() == WL_CONNECTED){
        WiFi.disconnect();
      }
      iconWindow(tft, 30, 30, 740, 430, 15);
      break;
    // Decrease volume
    case 4:
      if(bluetoothActive){
        currentVolume = a2dp_sink.get_volume();
        if (currentVolume > 10) {
          a2dp_sink.set_volume(currentVolume - 10);
          // button flashing code //
          iconPressButton(tft, 40, 365, 220, 100);  // flash the button 
          delay(20);
          iconButton(tft, 40, 365, 220, 100);     // SW4
          tft.textMode();
          tft.textTransparent(BROWN);
          tft.setTextSize(5);
          tft.textEnlarge(3);
          tft.textSetCursor(138, 370);  
          tft.write(0x1F); // Down
          tft.setTextSize(1);
          tft.textEnlarge(0);
          tft.textSetCursor(105, 425);
          tft.print("Volume Down");
          // end button flashing code //
        }
      }
      break;
    case 5:
      xTimerStop(updateTimer, 0);
      timerActive = false;
      decreaseTimer();
      // button flashing code //
        iconPressButton(tft, 40, 365, 220, 100);  // flash the button 
        delay(20);
        iconButton(tft, 40, 365, 220, 100);     // SW4
        tft.textMode();
        tft.textTransparent(BROWN);
        tft.setTextSize(5);
        tft.textEnlarge(3);
        tft.textSetCursor(138, 370);  
        tft.write(0x1F); // Down
        tft.setTextSize(1);
        tft.textEnlarge(0);
        tft.textSetCursor(100, 425);
        tft.print("Decrease Time");
      // end button flashing code //
      break;
    default:
      break;
  }
}

// Button 5 Logic
void button5(int currdisp){
  switch(activeDisplayMode){
    case 2:
      if(WiFi.status() == WL_CONNECTED){
        WiFi.disconnect();
      }
      iconWindow(tft, 30, 30, 740, 430, 15);
      break;
    // Increase volume
    case 4: 
      if(bluetoothActive){
        currentVolume = a2dp_sink.get_volume();
        if (currentVolume < 127) {
          a2dp_sink.set_volume(currentVolume + 10);
          // button flashing code //
           tft.graphicsMode();
          iconPressButton(tft, 288, 365, 220, 100);  // flash the button 
          delay(20);
          tft.graphicsMode();
          iconButton(tft, 288, 365, 220, 100);    // SW5
          tft.textMode();
          tft.textTransparent(BROWN);
          tft.setTextSize(5);
          tft.textEnlarge(3);
          tft.textSetCursor(386, 370);  
          tft.write(0x1E); // Up
          tft.setTextSize(1);
          tft.textEnlarge(0);
          tft.textSetCursor(360, 425);
          tft.print("Volume Up");
          // end button flashing code //
        }
      }
      break;
    case 5:
      xTimerStop(updateTimer, 0);
      timerActive = false;
      increaseTimer();
      // button flashing code //
      tft.graphicsMode();
      iconPressButton(tft, 288, 365, 220, 100);  // flash the button 
      delay(20);
      tft.graphicsMode();
      iconButton(tft, 288, 365, 220, 100);    // SW5
      tft.textMode();
      tft.textTransparent(BROWN);
      tft.setTextSize(5);
      tft.textEnlarge(3);
      tft.textSetCursor(386, 370);  
      tft.write(0x1E); // Up
      tft.setTextSize(1);
      tft.textEnlarge(0);
      tft.textSetCursor(358, 425);
      tft.print("Increase Time");
      // end button flashing code //
      break;
    default:
      break;
  }
}

///*********************** DISPLAY MODE ***********************///

// Update Display Mode Task (sw1)
void DisplayTask(void *parameter) {
  int displayMode;
  while (true) {
    if(xQueueReceive(displayQueue, &displayMode, portMAX_DELAY) == pdPASS){
      Serial.println("button 1 pressed");
      xTimerStop(updateclock, 0);
      xTimerStop(updateTimer, 0);
      timerActive = false; 
      activeDisplayMode = displayMode;
      updateDefaultDisplay(activeDisplayMode);
      switch(activeDisplayMode){
        case 1:
          xTimerReset(updateclock, 0);
          WiFi.disconnect();
          Serial.println("wifi diconnected");
          break;
        case 2:
          xTimerStop(updateclock, 0);
          break;
        case 3:
          WiFi.disconnect();
          Serial.println("wifi diconnected");
          break;
        case 4:
          break;
        case 5:
          xTimerStop(updateclock, 0);
          if(bluetoothActive){
            xQueueSend(bluetoothQueue, &bluetoothActive, portMAX_DELAY);
          }
          break;
        default:
          break;
      }
    }
    vTaskDelay(100/ portTICK_RATE_MS);
  }
}

/// STATIC DISPLAY MODES ///

// Update display mode
void updateDefaultDisplay(int Mode){
    switch (Mode) {
      case 1:
        HomeDisplay();
        //drawBMP(0x200000, 200, 50);
        break;
      case 2:
        RemindersDisplay();
        break;
      case 3:
        NightLightDisplay();
        break;
      case 4:
        MusicDisplay();
        break;
      case 5:
        TimerDisplay();
        break;
      default:
        break;
    }
}

// Home display mode
void HomeDisplay(){
  Serial.println("Home display");

  // basic information in left panel //
  tft.fillScreen(BROWN);
  tft.textMode();
  tft.textSetCursor(30, 10);
  tft.textTransparent(RA8875_WHITE);
  tft.textEnlarge(3);

  tft.print(rtc.getTime());

  tft.textSetCursor(90, 80);
  tft.textEnlarge(1);
 
  tft.print(rtc.getTime("%A"));
  tft.textSetCursor(95, 115);
  tft.print(rtc.getTime("%B %d"));

  // DRAW WEATHER ICON //

  iconWindow(tft, 30, 200, 250, 260, 25);
  iconCloudy(tft, 50, 225, 175);
  // drawWeather();

  // END DRAW WEATHER ICON // 

  tft.textMode();
  tft.textTransparent(BROWN);
  tft.textSetCursor(60, 400);
  tft.textEnlarge(1);

  tft.print(currentTemp);
  tft.write(0x09);
  tft.print("F, ");
  tft.print(currentWeather);

  // navigation aids in middle panel //
  tft.textMode();
  iconButton(tft, 320, 360, 220, 100);
  tft.setTextSize(5);
  tft.textEnlarge(3);
  tft.textTransparent(BROWN);
  tft.textSetCursor(418, 365); 
  tft.write(0x1A); // arrow
  tft.setTextSize(1);
  tft.textEnlarge(0);
  tft.textSetCursor(375, 420);
  tft.print("Switch Display");

  iconButton(tft, 560, 360, 220, 100); // set buttons x+240 away from next one
  iconCloud(tft, 670, 400, 10);
  tft.setTextSize(1);
  tft.textEnlarge(0);
  tft.textTransparent(BROWN);
  tft.textSetCursor(645, 420);
  tft.print("Weather");

  tft.fillRect(390, 10, 320, 320, LIGHT_BROWN);
  tft.fillRect(400, 20, 300, 300, LIGHTER_BROWN);
  tft.graphicsMode();
  drawBMP(EELOGO, 410, 30);
}

// Reminders display mode
void RemindersDisplay(){
  tft.fillScreen(BROWN);
  iconWindow(tft, 30, 30, 740, 430, 15);
  // tft.fillRoundRect(30, 30, 740, 430, 15, LIGHT_BROWN);

  // Display text on RA8875 TFT LCD
  tft.textMode();
  tft.setTextSize(1);
  tft.textEnlarge(1);
  tft.textSetCursor(50, 70);
  tft.textTransparent(RA8875_WHITE);
  tft.print("Upload notes by pressing button 2");
}

// Nightlight display mode
void NightLightDisplay() {
  Serial.println("Night light");
  tft.fillScreen(RA8875_BLACK);
}

// Music display mode
void MusicDisplay() {
  Serial.println("Music Display");
  tft.fillScreen(BROWN);

  // Buttons
  // set buttons x+248 away from next one
  iconButton(tft, 40, 245, 220, 100);     // SW1
  iconButton(tft, 288, 245, 220, 100);    // SW2
  iconButton(tft, 536, 245, 220, 100);    // SW3
  iconButton(tft, 40, 365, 220, 100);     // SW4
  iconButton(tft, 288, 365, 220, 100);    // SW5
  iconButton(tft, 536, 365, 220, 100);    // SW6

  // Button Symbols
  tft.textMode();  
  tft.setTextSize(5);
  tft.textEnlarge(3);
  tft.textTransparent(BROWN);

  // SW1 (Display Mode)
  tft.textSetCursor(138, 250);  // (x + 98) and (y + 5)
  tft.write(0x1A); // arrow

  // SW2 (Pairing)
  tft.textSetCursor(386, 250);  // (x + 98) and (y + 5)
  tft.write(0x0E); // Music Symbol

  // SW3 (Play/Pause)
  tft.textSetCursor(634, 250); 
  tft.write(0x10); // Play

  // SW4
  tft.textSetCursor(138, 370);  // (x + 98) and (y + 5)
  tft.write(0x1F); // Down

  // SW5
  tft.textSetCursor(386, 370);  // (x + 98) and (y + 5)
  tft.write(0x1E); // Up

  // SW6 (Switch Display)
  tft.textSetCursor(634, 370); 
  tft.write(0x1B); // arrow

  // Button Descriptions
  tft.setTextSize(1);
  tft.textEnlarge(0);
  tft.textTransparent(BROWN);

  // SW1
  tft.textSetCursor(95, 305);
  tft.print("Switch Display");

  // SW2
  tft.textSetCursor(375, 310);
  tft.print("BT Off");

  // SW3
  tft.textSetCursor(630, 305);
  tft.print("Play");

  // SW4
  tft.textSetCursor(105, 425);
  tft.print("Volume Down");

  // SW5
  tft.textSetCursor(360, 425);
  tft.print("Volume Up");

  // SW6
  tft.textSetCursor(591, 425);
  tft.print("Switch Display");

  iconPopBox(tft, 63, 3, 674, 234, 3);

  tft.graphicsMode();
  drawBMP(MUSIC, 80, 20); // 640 x 200
}

// Timer display mode
void TimerDisplay(){
  tft.fillScreen(BROWN);

  // Buttons
  // set buttons x+248 away from next one
  iconButton(tft, 40, 245, 220, 100);     // SW1
  iconButton(tft, 288, 245, 220, 100);    // SW2
  iconButton(tft, 536, 245, 220, 100);    // SW3
  iconButton(tft, 40, 365, 220, 100);     // SW4
  iconButton(tft, 288, 365, 220, 100);    // SW5
  iconButton(tft, 536, 365, 220, 100);    // SW6

  // Button Symbols
  tft.textMode();  
  tft.setTextSize(5);
  tft.textEnlarge(3);
  tft.textTransparent(BROWN);

  // SW1 (Display Mode)
  tft.textSetCursor(138, 250);  // (x + 98) and (y + 5)
  tft.write(0x1A); // arrow

  // SW2 (Start/Stop)
  tft.textSetCursor(386, 250);  // (x + 98) and (y + 5)
  tft.write(0x10); // Play

  // SW3 (Play/Pause)
  tft.textSetCursor(634, 250); 
  tft.write(0x2A); // Reset

  // SW4
  tft.textSetCursor(138, 370);  // (x + 98) and (y + 5)
  tft.write(0x1F); // Down

  // SW5
  tft.textSetCursor(386, 370);  // (x + 98) and (y + 5)
  tft.write(0x1E); // Up

  // SW6 (Switch Display)
  tft.textSetCursor(634, 370); 
  tft.write(0x1B); // arrow

  // Button Descriptions
  tft.setTextSize(1);
  tft.textEnlarge(0);
  tft.textTransparent(BROWN);

  // SW1
  tft.textSetCursor(95, 305);
  tft.print("Switch Display");

  // SW2
  tft.textSetCursor(382, 305);
  tft.print("Play");

  // SW3
  tft.textSetCursor(628, 305);
  tft.print("Reset");

  // SW4
  tft.textSetCursor(100, 425);
  tft.print("Decrease Time");

  // SW5
  tft.textSetCursor(358, 425);
  tft.print("Increase Time");

  // SW6
  tft.textSetCursor(591, 425);
  tft.print("Switch Display");


  remainingSeconds = user_min * 60;
  minutes = remainingSeconds / 60;
  seconds = remainingSeconds % 60;

  tft.textMode();
  tft.textSetCursor(250, 40);
  tft.textEnlarge(2);
  tft.textTransparent(RA8875_WHITE);
  tft.print("Start Timer");

  char timeStr[6];
  sprintf(timeStr, "%02d:%02d", minutes, seconds);
  tft.textSetCursor(300, 120);
  tft.textEnlarge(3);
  tft.print(timeStr);
  // END STATIC "START TIMER" MESSAGE //

  timerActive = false; // Timer does not start by default
}

///**************** TIMER INTERRUPTS *****************///

// Timer interrupt for clock update
void updateClock(TimerHandle_t xTimer){
  // Clear the time area (implement this as needed for your display)
  // For example, you might fill a rectangle with the background color over the time
  tft.fillRect(15, 10, 268, 65, BROWN);

  // Print the new time
  tft.textMode();
  tft.textTransparent(RA8875_WHITE);
  tft.textSetCursor(20, 10); // Reset cursor position if needed
  tft.setTextSize(1);
  tft.textEnlarge(3);

  if(checkTimeinfo()){
    connectToWiFi();
    getLocalTime(&timeinfo);
    weatherAPI();
    WiFi.disconnect();
    rtc.setTimeStruct(timeinfo);
  } else {
    tft.print(rtc.getTime());
  }

  if(activeDisplayMode == 1){
    xTimerReset(updateclock, 0);
  }
}

// Dynamically update time left
void updateTimerDisplay(TimerHandle_t xTimer) {
  
  remainingSeconds--;
  minutes = remainingSeconds / 60;
  seconds = remainingSeconds % 60;

  // DYNAMIC "TIME LEFT" MESSAGE //
  tft.fillRect(295, 115, 170, 70, BROWN); // CLEAR RECTANGLE OF LCD TO REPRINT TIME LEFT
  tft.textMode();
  tft.textTransparent(RA8875_WHITE);
  char timeStr[6];
  sprintf(timeStr, "%02d:%02d", minutes, seconds);
  tft.textSetCursor(300, 120);
  tft.textEnlarge(3);
  tft.print(timeStr);
  // END DYNAMIC "TIME LEFT" MESSAGE //

  if((timerActive) && (remainingSeconds > 0)){
    xTimerReset(updateTimer, 0);
  } else if ((timerActive) && (remainingSeconds <= 0)){
    xTimerStop(updateTimer,0);
    timerDone(remainingSeconds, minutes, seconds);
  }
}

///********************* TIMER FUNCTIONS *************************///

// Start the timer (from whever it left off)
void startTimer(int &remainingSeconds, int &minutes, int &seconds){
  Serial.println("Timer started!");

  minutes = remainingSeconds / 60;
  seconds = remainingSeconds % 60;   

  tft.fillRect(250, 40, 290, 175, BROWN);
  // DYNAMIC "TIME LEFT" MESSAGE //
  tft.textMode();
  tft.textSetCursor(265, 40);
  tft.setTextSize(1);
  tft.textEnlarge(2);
  tft.textTransparent(RA8875_WHITE);
  tft.print("Time Left");
  char timeStr[6];
  sprintf(timeStr, "%02d:%02d", minutes, seconds);
  tft.textSetCursor(300, 120);
  tft.textEnlarge(3);
  tft.print(timeStr);
  // END DYNAMIC "TIME LEFT" MESSAGE //
  timerActive = true;
  if((timerActive) && (remainingSeconds > 0)){
    xTimerReset(updateTimer, 0);
  }
}

// Stop the timer
void stopTimer(int &remainingSeconds, int &minutes, int &seconds){

  Serial.println("Timer paused!");


  minutes = remainingSeconds / 60;
  seconds = remainingSeconds % 60;

  tft.fillRect(250, 40, 290, 175, BROWN);
  tft.textMode();
  tft.textSetCursor(265, 40);
  tft.setTextSize(1);
  tft.textEnlarge(2);
  tft.textTransparent(RA8875_WHITE);
  tft.print("Time Paused");
  char timeStr[6];
  sprintf(timeStr, "%02d:%02d", minutes, seconds);
  tft.textSetCursor(300, 120);
  tft.textEnlarge(3);
  tft.print(timeStr);

  timerActive = false; 
}

// When the timer is done
void timerDone(int &remainingSeconds,int &minutes, int &seconds){
    // update seconds one last time to get to 0
  seconds = remainingSeconds % 60;
  Serial.println("Time up!");
  tft.fillRect(250, 40, 290, 175, BROWN);
  // STATIC "TIMER DONE" MESSAGE //
  tft.textMode();
  tft.textSetCursor(260, 40);
  tft.setTextSize(1);
  tft.textEnlarge(2);
  tft.textTransparent(RA8875_WHITE);
  tft.print("Timer Done");
  char timeStr[6];
  sprintf(timeStr, "%02d:%02d", minutes, seconds);
  tft.textSetCursor(300, 120);
  tft.textEnlarge(3);
  tft.print(timeStr);
  // END STATIC "TIMER DONE" MESSAGE //
  timerActive = false; // Stop the timer
}

void increaseTimer(){     
  if (user_min < 58) {
    user_min = user_min + 1;
  } else {
    user_min = 1;
  }
  remainingSeconds = user_min * 60;
  minutes = remainingSeconds / 60;
  seconds = remainingSeconds % 60;

  tft.fillRect(250, 40, 290, 175, BROWN);
  tft.textMode();
  tft.textSetCursor(250, 40);
  tft.setTextSize(1);
  tft.textEnlarge(2);
  tft.textTransparent(RA8875_WHITE);
  tft.print("Start Timer");

  char timeStr[6];
  sprintf(timeStr, "%02d:%02d", minutes, seconds);
  tft.textSetCursor(300, 120);
  tft.textEnlarge(3);
  tft.print(timeStr);
}

void decreaseTimer(){
  if (user_min > 1){
    user_min = user_min - 1;
  } else {
    user_min = 59;
  }
  remainingSeconds = user_min * 60;
  minutes = remainingSeconds / 60;
  seconds = remainingSeconds % 60;

  tft.fillRect(250, 40, 290, 175, BROWN);
  tft.textMode();
  tft.textSetCursor(250, 40);
  tft.setTextSize(1);
  tft.textEnlarge(2);
  tft.textTransparent(RA8875_WHITE);
  tft.print("Start Timer");

  char timeStr[6];
  sprintf(timeStr, "%02d:%02d", minutes, seconds);
  tft.textSetCursor(300, 120);
  tft.textEnlarge(3);
  tft.print(timeStr);
}

void timerReset(){
  remainingSeconds = user_min * 60;
  minutes = remainingSeconds / 60;
  seconds = remainingSeconds % 60;

  tft.fillRect(250, 40, 290, 175, BROWN);
  tft.textMode();
  tft.textSetCursor(250, 40);
  tft.setTextSize(1);
  tft.textEnlarge(2);
  tft.textTransparent(RA8875_WHITE);
  tft.print("Start Timer");

  char timeStr[6];
  sprintf(timeStr, "%02d:%02d", minutes, seconds);
  tft.textSetCursor(300, 120);
  tft.setTextSize(1);
  tft.textEnlarge(3);
  tft.print(timeStr);
  Serial.println("timer reset");
}

///************************* WIFI ****************************///

// Connect to WiFi
void connectToWiFi(){
  WiFi.begin(ssid,password);
  Serial.println("Connecting to WiFi");
  while (WiFi.status() != WL_CONNECTED) {
    delay(1000);
    Serial.println("...");
  }
  Serial.println("Connected to WiFi");
}

// Check if we need to update the time from WiFi
bool checkTimeinfo(){
  bool timechecked = false;
  if((timeCheck + 30) > 60){
    if(((timeCheck + 30) - 60) == rtc.getMinute()){
      timeCheck = rtc.getMinute();
      Serial.println("time checked");
      timechecked = true;
    } 
  } else if ((timeCheck + 30) == rtc.getMinute()) {
    if((timeCheck + 30) == rtc.getMinute()){
      timeCheck = rtc.getMinute();
      Serial.println("time checked");
      timechecked = true;
    }
  } else {
      timechecked = false;
  }
  return timechecked;
}

void updateWeatherCondition(const String& newCondition, const String& newIcon) {
    currentWeather = newCondition;
    currentIcon = newIcon;
    // conditions programmed into the switch case are listed below //
    // "clear" & "01d" (day condition for clear)
    // "clear" & "01n" (night condition for clear)
    // "clouds" & "02d" (day condition)
    // "clouds" & "02n" (night condition)
    // "clouds" & not "02d or 02n" (day or night condition)
    // "rain" (day or night condition)
    // "snow" (day or night condition)
}

void drawWeather() {
    if (currentWeather == "Clear") {
      if(currentIcon == "01d"){
        iconClearDay(tft, 170, 240, 50);
      } else if (currentIcon == "01n"){
        iconClearNight(tft, 100, 150, 175);
      }
    } else if (currentWeather == "Clouds") {
      if(currentIcon == "02d"){
        iconCloudyDay(tft, 65, 225, 175);
      } else if (currentIcon == "02n"){
         iconCloudyNight(tft, 65, 225, 175);
      } else if (currentIcon == "03d"){
         iconCloudy(tft, 50, 225, 175);
      } else if (currentIcon == "03n"){
         iconCloudy(tft, 50, 225, 175);
      } else {
        iconCloudyDay(tft, 65, 225, 175);
      }
    } else if (currentWeather == "Rain" || currentWeather == "Drizzle") {
        iconRain(tft, 60, 150, 175);
    } else if (currentWeather =="Thunderstorm"){
      iconThunderstorm(tft, 50, 240, 175);
    } else if (currentWeather == "Snow") {
        iconSnow(tft, 100, 150, 175);
    } else {
        iconSun(tft, 160, 315, 40);
    }
}

void weatherAPI(){
  if(WiFi.status()== WL_CONNECTED){

    jsonBuffer = httpGETRequest(serverPath.c_str());

    // Parse response
    StaticJsonDocument<300> filter;
    filter["weather"][0]["main"] = true;
    filter["weather"][0]["icon"] = true;
    filter["main"]["temp"] = true;

    StaticJsonDocument<900> doc;

    DeserializationError error = deserializeJson(doc, jsonBuffer);
    deserializeJson(doc, jsonBuffer, DeserializationOption::Filter(filter));

    // Test if parsing succeeds.
    if (error) {
      Serial.print(F("deserializeJson() failed: "));
      Serial.println(error.f_str());
      return;
    }

    JsonObject obj = doc.as<JsonObject>();


    String weather = obj[String("weather")][0][String("main")];
    String icon = obj[String("weather")][0][String("icon")];
    float rawtemp = obj[String("main")][String("temp")];

    updateWeatherCondition(weather, icon);

    // Convert raw temp to fahrenheit
    float tempFloat = (rawtemp - 273.15)*1.8 + 32;
    currentTemp = (int)tempFloat;
    
    } else {
      Serial.println("WiFi Disconnected");
    }
}

String httpGETRequest(const char* serverName) {
  WiFiClient client;
  HTTPClient http;
    
  // Your Domain name with URL path or IP address with path
  http.begin(client, serverName);
  
  // Send HTTP POST request
  int httpResponseCode = http.GET();
  
  String payload = "{}"; 
  
  if (httpResponseCode>0) {
    Serial.print("HTTP Response code: ");
    Serial.println(httpResponseCode);
    payload = http.getString();
  }
  else {
    Serial.print("Error code: ");
    Serial.println(httpResponseCode);
  }
  // Free resources
  http.end();

  return payload;
}

///************************* BLUETOOTH *****************************///
void handleBluetoothCommands(void* Parameter) {
  while (true) {
    if (xQueueReceive(bluetoothQueue, &bluetoothActive, portMAX_DELAY) == pdTRUE) {
      if(bluetoothActive){
        a2dp_sink.end();
        Serial.println("Bluetooth deactived");
        bluetoothActive = false;
        bluetoothPlay = false;
      } else if (!bluetoothActive){
        a2dp_sink.set_i2s_config(i2s_config2);
        a2dp_sink.set_i2s_port(I2S_NUM_0);
        a2dp_sink.set_pin_config(pin_config2);
        a2dp_sink.start("ESP32_Bluetooth");
        Serial.println("Bluetooth Actived");
        bluetoothActive = true;
      }
    } 
    vTaskDelay(100 / portTICK_PERIOD_MS);
  } 
}

///************************ DISPLAY NOTES **************************///
void handleRoot() {
  String htmlContent = "<!DOCTYPE html>\
                        <html>\
                        <head><title>PIXIE NOTES</title>\
                        <link href='https://fonts.googleapis.com/css2?family=Pixelify+Sans:wght@400&display=swap' rel='stylesheet'>\
                        <style>\
                          body { background-color: #CCAC; }\
                          h1 { font-family: 'Pixelify Sans', sans-serif; color: #72E8; font-size: 24px; text-align: center; }\
                          .form-container { display: flex; flex-direction: column; align-items: center; }\
                          textarea { color: black; background-color: white; width: 90%; height: 150px; padding: 8px; border: 2px solid #72E8; margin-bottom: 10px; }\
                          input[type='submit'] { \
                            font-family: 'Pixelify Sans', sans-serif; \
                            width: 20%; \
                            padding: 8px 15px; \
                            border: none; \
                            background-color: #72E8; \
                            color: white; \
                            cursor: pointer; \
                          }\
                        </style>\
                        </head>\
                        <body>\
                        <h1>PIXIE NOTES</h1>\
                        <div class='form-container'>\
                          <form action='/upload' method='post'>\
                            <textarea name='text'></textarea>\
                            <input type='submit' value='Submit'>\
                          </form>\
                        </div>\
                        </body>\
                        </html>";

  server.send(200, "text/html", htmlContent);
}

void handleUpload() {
  if (server.hasArg("text")) {
    String text = server.arg("text");
    Serial.println(text);

    // Clear screen
    tft.fillRoundRect(30, 30, 740, 430, 15, LIGHT_BROWN);

    // Display text on RA8875 TFT LCD
    tft.textMode();
    tft.setTextSize(1);
    tft.textEnlarge(1);
    tft.textSetCursor(40, 40);
    tft.textTransparent(RA8875_WHITE);
    tft.print(text);

    // Send response to client
    server.send(200, "text/html", "Text received: <b>" + text + "</b><br>Press button 2 to upload more notes.");
    received = true;
  } else {
    server.send(404, "text/plain", "Bad Request");
    received = true;
  } 
}